/**
 * Demonstrates using the add/go/get method to efficiently write and read
 * some or all analog and digital I/O on the LabJack U3.
 * Records the time for 100 iterations and divides by 100, to allow
 * verification of the basic command/response communication times of the
 * LabJack U3 as documented in Section 3.1 of the U3 User's Guide.
 *
 * support@labjack.com
 */

import java.util.Arrays;
import com.sun.jna.*;
import com.sun.jna.ptr.*;
import com.labjack.LJUD;
import com.labjack.LJUDException;

public class AllIO {

	public AllIO() {
	}
	
	private void handleLJUDException(LJUDException e) {
		e.printStackTrace();
		if(e.getError() > LJUD.Errors.MIN_GROUP_ERROR.getValue()) {
			System.exit(-1);
		}
	}

	//Displays warning message if there is one. Error values < 0 are warnings
	//and do not cause a LJUDException in the LJUD class.
	private void checkForWarning(int error) {
		Pointer errorStringPtr = new Memory(256);
		if(error < 0) {
			LJUD.errorToString(error, errorStringPtr);
			System.out.println("Warning: " + errorStringPtr.getString(0).trim());
		}
	}
	
	public void runExample() {
		try {
			int intErrorcode = 0;
			int intHandle = 0;
			IntByReference refHandle = new IntByReference(0);
			boolean isDone = false;

			IntByReference refIOType = new IntByReference(0);
			IntByReference refChannel = new IntByReference(0);
			DoubleByReference refValue = new DoubleByReference(0);
			int intIOType = 0;
			int intChannel = 0;
			double dblValue = 0.0;
			
			long time=0;
			int numIterations = 100;
			final int numChannels = 16;  //Number of AIN channels, 0-16.
			int quickSample = 0;  //Set to TRUE for quick AIN sampling.
			int intSettling = 1;  //Set to TRUE for extra AIN settling time.

			double valueDIPort = 0;
			double[] valueAIN = new double[numChannels];
			Arrays.fill(valueAIN, 9999.0);

			// Variables to satisfy certain method signatures
			IntByReference dummyInt = new IntByReference(0);
			DoubleByReference dummyDouble = new DoubleByReference(0.0);

			//Open the first found LabJack.
			intErrorcode = LJUD.openLabJack(LJUD.Constants.dtU3, LJUD.Constants.ctUSB, "1", 1, refHandle);
			checkForWarning(intErrorcode);
			intHandle = refHandle.getValue();
			
			//Start by using the pin_configuration_reset IOType so that all
			//pin assignments are in the factory default condition.
			LJUD.ePut(intHandle, LJUD.Constants.ioPIN_CONFIGURATION_RESET, 0, 0, 0);

			//Configure quickSample.
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chAIN_RESOLUTION, quickSample, 0);

			//Configure longSettling.
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chAIN_SETTLING_TIME, intSettling, 0);

			//Configure the necessary lines as analog.
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_ANALOG_ENABLE_PORT, 0, Math.pow(2,numChannels)-1, numChannels);
		
			//Now an Add/Go/Get block to configure the timers and counters.  These
			//are configured on EIO0-EIO3, so if more than 8 analog inputs are
			//enabled then the analog inputs use these lines.
			if(numChannels <= 8) {
				//Set the timer/counter pin offset to 8, which will put the first
				//timer/counter on EIO0.
				LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chTIMER_COUNTER_PIN_OFFSET, 8, 0, 0);
				
				//Use the default clock source.
				LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chTIMER_CLOCK_BASE, LJUD.Constants.tc48MHZ, 0, 0);
				
				//Enable 2 timers.
				LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chNUMBER_TIMERS_ENABLED, 2, 0, 0);
				
				//Configure Timer0 as 8-bit PWM.
				LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_TIMER_MODE, 0, LJUD.Constants.tmPWM8, 0, 0);
				
				//Set the PWM duty cycle to 50%.
				LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_TIMER_VALUE, 0, 32768, 0, 0);

				//Configure Timer1 as 8-bit PWM.
				LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_TIMER_MODE, 1, LJUD.Constants.tmPWM8, 0, 0);
				
				//Set the PWM duty cycle to 50%.
				LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_TIMER_VALUE, 1, 32768, 0, 0);

				//Enable Counter0.
				LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_COUNTER_ENABLE, 0, 1, 0, 0);

				// Enable Counter1
				LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_COUNTER_ENABLE, 1, 1, 0, 0);

				//Execute the requests.
				LJUD.goOne(intHandle);
			}

			//Now add requests that will be processed every iteration of the loop.

			//Add analog input requests.
			for(int j = 0; j < numChannels; j++) {
				LJUD.addRequest(intHandle, LJUD.Constants.ioGET_AIN, j, 0, 0, 0);
			}

			//Set DAC0 to 2.5 volts.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_DAC, 0, 2.5, 0, 0);
			
			//Read CIO digital lines.
			LJUD.addRequest(intHandle, LJUD.Constants.ioGET_DIGITAL_PORT, 16, 0, 4, 0);
			
			//Only do the timer/counter stuff if there are less than 8 analog inputs.
			if(numChannels <= 8) {
				LJUD.addRequest(intHandle, LJUD.Constants.ioGET_COUNTER, 0, 0, 0, 0);
				LJUD.addRequest(intHandle, LJUD.Constants.ioGET_COUNTER, 1, 0, 0, 0);
				LJUD.addRequest(intHandle, LJUD.Constants.ioGET_TIMER, 0, 0, 0, 0);
				LJUD.addRequest(intHandle, LJUD.Constants.ioGET_TIMER, 1, 0, 0, 0);

				//Set the PWM duty cycle to 50%.
				LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_TIMER_VALUE, 0, 32768, 0, 0);

				//Set the PWM duty cycle to 50%.
				LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_TIMER_VALUE, 1, 32768, 0, 0);
			}


			time = System.currentTimeMillis();	

			for(int i = 0; i < numIterations; i++) {
				//Execute the requests.
				LJUD.goOne(intHandle);

				//Get all the results.  The input measurement results are stored.  All other
				//results are for configuration or output requests so we are just checking
				//whether there was an error.
				LJUD.getFirstResult(intHandle, refIOType, refChannel, refValue, dummyInt, dummyDouble);

				// Get results until there is no more data available
				isDone = false;
				while(!isDone) {
					try {
						intIOType = refIOType.getValue();
						intChannel = refChannel.getValue();
						dblValue = refValue.getValue();
						switch(intIOType) {
							case LJUD.Constants.ioGET_AIN:
								valueAIN[intChannel] = dblValue;
								break;
							case LJUD.Constants.ioGET_DIGITAL_PORT:
								valueDIPort = dblValue;
								break;
							default:
								break;
						}

						LJUD.getNextResult(intHandle, refIOType, refChannel, refValue, dummyInt, dummyDouble);
					}
					catch(LJUDException le) {
						if(le.getError() == LJUD.Errors.NO_MORE_DATA_AVAILABLE.getValue()) {
							isDone = true;
						}
						else {
							throw le;
						}
					}
				}
			}

			time = System.currentTimeMillis() - time;

			System.out.println("Milliseconds per iteration = " + ((double)time/(double)numIterations));
			System.out.println("\nDigital Input = " + valueDIPort);
			System.out.println("\nAIN readings from last iteration:\n");
			
			for(int j = 0; j < numChannels; j++) {
				System.out.println(valueAIN[j]);
			}
		}
		catch(LJUDException le) {
			handleLJUDException(le);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		new AllIO().runExample();
	}
}
