/**
 * Demonstrates the E-functions with the LabJack U3.  For timer/counter
 * testing, connect FIO4 to FIO5 and FIO6.
 *
 * support@labjack.com
 */

import com.sun.jna.*;
import com.sun.jna.ptr.*;
import com.labjack.LJUD;
import com.labjack.LJUDException;

public class EFunctions {

	public EFunctions() {
	}

	private void handleLJUDException(LJUDException e) {
		e.printStackTrace();
		if(e.getError() > LJUD.Errors.MIN_GROUP_ERROR.getValue()) {
			System.exit(-1);
		}
	}

	//Displays warning message if there is one. Error values < 0 are warnings
	//and do not cause a LJUDException in the LJUD class.
	private void checkForWarning(int error) {
		Pointer errorStringPtr = new Memory(256);
		if(error < 0) {
			LJUD.errorToString(error, errorStringPtr);
			System.out.println("Warning: " + errorStringPtr.getString(0).trim());
		}
	}

	public void runExample() {
		try {
			int intErrorcode;
			double dblValue = 0.0;
			DoubleByReference refDblValue = new DoubleByReference(0.0);
			int intValue = 0;
			IntByReference refIntValue = new IntByReference(0);
			int intHandle = 0;
			IntByReference refHandle = new IntByReference(0);
			int resolution = 0;
			int settling = 0;
			int binary = 0;
			int[] aEnableTimers = new int[2];
			int[] aEnableCounters = new int[2];
			int TCPinOffset, timerClockBaseIndex, timerClockDivisor;
			int[] aTimerModes = new int[2];
			double[] adblTimerValues = new double[2];
			int[] aReadTimers = new int[2];
			int[] aUpdateResetTimers = new int[2];
			int[] aReadCounters = new int[2];
			int[] aResetCounters = new int[2];
			double[] adblCounterValues = new double[2];
			double highTime, lowTime, dutyCycle;
	
			//Open the first found LabJack U3.
			intErrorcode = LJUD.openLabJack(LJUD.Constants.dtU3, LJUD.Constants.ctUSB, "1", 1, refHandle);
			checkForWarning(intErrorcode);
			intHandle = refHandle.getValue();
			
			//Start by using the pin_configuration_reset IOType so that all
			//pin assignments are in the factory default condition.
			LJUD.ePut(intHandle, LJUD.Constants.ioPIN_CONFIGURATION_RESET, 0, 0, 0);
	
			//Take a single-ended measurement from AIN3.
			resolution = 0; //A nonzero value to enable QuickSample.
			settling = 0; //A nonzero value to enable LongSettling.
			binary = 0;
			LJUD.eAIN(intHandle, 3, 31, refDblValue, 0, resolution, settling, binary, 0, 0);
			System.out.println("AIN3 = " + refDblValue.getValue());
	
			//Set DAC0 to 3.0 volts.
			dblValue = 3.0;
			binary = 0;
			LJUD.eDAC(intHandle, 0, dblValue, binary, 0, 0);
			System.out.println("DAC0 set to " + dblValue + " volts");
	
			//Read state of FIO4.
			LJUD.eDI(intHandle, 4, refIntValue);
			System.out.println("FIO4 = " + refIntValue.getValue());
	
			//Set the state of FIO7.
			intValue = 1;
			LJUD.eDO(intHandle, 7, intValue);
			System.out.println("FIO7 set to = " + intValue + "\n");
	
			//Timers and Counters example.
			//First, a call to eTCConfig. Fill the arrays with the desired
			//values, then make the call.
			aEnableTimers[0] = 1;  //Enable Timer0 (uses FIO4).
			aEnableTimers[1] = 1;  //Enable Timer1 (uses FIO5).
			aEnableCounters[0] = 0;  //Disable Counter0.
			aEnableCounters[1] = 1;  //Enable Counter1 (uses FIO6).
			TCPinOffset = 4;  //Offset is 4, so timers/counters start at FIO4.
			timerClockBaseIndex = LJUD.Constants.tc48MHZ_DIV;  //Base clock is 48 MHz with divisor support, so Counter0 is not available.
			//timerClockBaseIndex = LJUD.Constants.tc24MHZ_DIV;  //Use this line instead for hardware rev 1.20.
			timerClockDivisor = 48;  //Thus timer clock is 1 MHz.
			//timerClockDivisor = 24;  //Use this line instead for hardware rev 1.20.
			aTimerModes[0] = LJUD.Constants.tmPWM8;	 //Timer0 is 8-bit PWM output. Frequency is 1M/256 = 3906 Hz.
			aTimerModes[1] = LJUD.Constants.tmDUTYCYCLE;  //Timer1 is duty cyle input.
			adblTimerValues[0] = 16384;  //Set PWM8 duty-cycle to 75%.
			adblTimerValues[1] = 0;
			LJUD.eTCConfig(intHandle, aEnableTimers, aEnableCounters,
					TCPinOffset, timerClockBaseIndex, timerClockDivisor,
					aTimerModes, adblTimerValues, 0, 0);
			System.out.println("Timers and Counters enabled.\n");
	
			Thread.sleep(1000); // Wait 1 second.
	
			//Now, a call to eTCValues.
			aReadTimers[0] = 0;  //Don't read Timer0 (output timer).
			aReadTimers[1] = 1;  //Read Timer1.
			aUpdateResetTimers[0] = 1;  //Update Timer0.
			aUpdateResetTimers[1] = 1;  //Reset Timer1.
			aReadCounters[0] = 0;
			aReadCounters[1] = 1;  //Read Counter1.
			aResetCounters[0] = 0;
			aResetCounters[1] = 1;  //Reset Counter1.
			adblTimerValues[0] = 32768;  //Change Timer0 duty-cycle to 50%.
			adblTimerValues[1] = 0;
			LJUD.eTCValues(intHandle, aReadTimers, aUpdateResetTimers,
					aReadCounters, aResetCounters, adblTimerValues,
					adblCounterValues, 0, 0);
			System.out.println("Timer1 value = " + adblTimerValues[1]);
			System.out.println("Counter1 value = "	+ adblCounterValues[1]);
	
			//Convert Timer1 value to duty-cycle percentage.
			//High time is LSW
			highTime = (double) (((long) adblTimerValues[1]) % (65536));
			//Low time is MSW
			lowTime = (double) (((long) adblTimerValues[1]) / (65536));
			//Calculate the duty cycle percentage.
			dutyCycle = (((highTime + lowTime) != 0) ? (100 * highTime / (highTime + lowTime)) : 0);
			System.out.println("\nHigh clicks Timer1 = " + highTime);
			System.out.println("Low clicks Timer1 = " + lowTime);
			System.out.println("Duty cycle Timer1 = " + dutyCycle);
	
			//Disable all timers and counters.
			aEnableTimers[0] = 0;
			aEnableTimers[1] = 0;
			aEnableCounters[0] = 0;
			aEnableCounters[1] = 0;
			LJUD.eTCConfig(intHandle, aEnableTimers, aEnableCounters, 4,
					timerClockBaseIndex, timerClockDivisor, aTimerModes,
					adblTimerValues, 0, 0);
		}
		catch(LJUDException le) {
			handleLJUDException(le);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		new EFunctions().runExample();
	}
}
