/*
 * Demonstrates talking to 1 or 2 EI-1050 probes. *
 *
 * support@labjack.com
 */

import com.sun.jna.*;
import com.sun.jna.ptr.*;
import com.labjack.LJUD;
import com.labjack.LJUDException;

public class EI1050 {

	public EI1050() {
	}

	private void handleLJUDException(LJUDException e) {
		e.printStackTrace();
		if(e.getError() > LJUD.Errors.MIN_GROUP_ERROR.getValue()) {
			System.exit(-1);
		}
	}

	//Displays warning message if there is one. Error values < 0 are warnings
	//and do not cause a LJUDException in the LJUD class.
	private void checkForWarning(int error) {
		Pointer errorStringPtr = new Memory(256);
		if(error < 0) {
			LJUD.errorToString(error, errorStringPtr);
			System.out.println("Warning: " + errorStringPtr.getString(0).trim());
		}
	}

	public void runExample() {
		try {
			int intErrorcode;
			DoubleByReference refValue = new DoubleByReference(0.0);
			double dblValue = 0.0;
			IntByReference refHandle = new IntByReference(0);
			int intHandle = 0;
	
			//Open the first found LabJack U3.
			intErrorcode = LJUD.openLabJack(LJUD.Constants.dtU3, LJUD.Constants.ctUSB, "1", 1, refHandle);
			checkForWarning(intErrorcode);
			intHandle = refHandle.getValue();
	
			//Start by using the pin_configuration_reset IOType so that all
			//pin assignments are in the factory default condition.
			LJUD.ePut(intHandle, LJUD.Constants.ioPIN_CONFIGURATION_RESET, 0, 0, 0);
	
			//Set the Data line to FIO4, which is the default anyway. 
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chSHT_DATA_CHANNEL, 4, 0);
	
			//Set the Clock line to FIO5, which is the default anyway. 
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chSHT_CLOCK_CHANNEL, 5, 0);
	
			//Set FIO6 to output-high to provide power to the EI-1050.
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_DIGITAL_BIT, 6, 1, 0);
		
		
			//Use this code if only a single EI-1050 is connected.
			//	Connections for one probe:
			//	Red (Power)         FIO6
			//	Black (Ground)      GND
			//	Green (Data)        FIO4
			//	White (Clock)       FIO5
			//	Brown (Enable)      FIO6
	
			//Now, an add/go/get block to get the temp and humidity at the same time.
			//Request a temperature reading from the EI-1050.
			LJUD.addRequest(intHandle, LJUD.Constants.ioSHT_GET_READING, LJUD.Constants.chSHT_TEMP, 0, 0, 0);
	
			//Request a humidity reading from the EI-1050.
			LJUD.addRequest(intHandle, LJUD.Constants.ioSHT_GET_READING, LJUD.Constants.chSHT_RH, 0, 0, 0);
	
			//Execute the requests.  Will take about 0.5 seconds with a USB high-high
			//or Ethernet connection, and about 1.5 seconds with a normal USB connection.
			LJUD.goOne(intHandle);
	
			//Get the temperature reading.
			LJUD.getResult(intHandle, LJUD.Constants.ioSHT_GET_READING, LJUD.Constants.chSHT_TEMP, refValue);
			dblValue = refValue.getValue();
			System.out.println("Temp Probe A = " + dblValue + " deg K");
			System.out.println("Temp Probe A = " + (dblValue - 273.15) + " deg C");
			System.out.println("Temp Probe A = " +  (((dblValue - 273.15) * 1.8) + 32) + " deg F");
	
			//Get the humidity reading.
			LJUD.getResult(intHandle, LJUD.Constants.ioSHT_GET_READING, LJUD.Constants.chSHT_RH, refValue);
			System.out.println("RH Probe A = " + refValue.getValue() + " percent\n");
	
			//End of single probe code.


			/*
			//Use this code if two EI-1050 probes are connected.
			//	Connections for both probes:
			//	Red (Power)         FIO6
			//	Black (Ground)      GND
			//	Green (Data)        FIO4
			//	White (Clock)       FIO5
			//
			//	Probe A:
			//	Brown (Enable)    FIO7
			//
			//	Probe B:
			//	Brown (Enable)    DAC0
	
			//Set FIO7 to output-low to disable probe A. 
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_DIGITAL_BIT, 7, 0, 0);
			
			//Set DAC0 to 0 volts to disable probe B.
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_DAC, 0, 0.0, 0);

			//Set FIO7 to output-high to enable probe A. 
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_DIGITAL_BIT, 7, 1, 0);
			
			//Now, an add/go/get block to get the temp  humidity at the same time.
			//Request a temperature reading from the EI-1050.
			LJUD.addRequest(intHandle, LJUD.Constants.ioSHT_GET_READING, LJUD.Constants.chSHT_TEMP, 0, 0, 0);
			
			//Request a humidity reading from the EI-1050.
			LJUD.addRequest(intHandle, LJUD.Constants.ioSHT_GET_READING, LJUD.Constants.chSHT_RH, 0, 0, 0);
			
			//Execute the requests.  Will take about 0.5 seconds with a USB high-high
			//or Ethernet connection, and about 1.5 seconds with a normal USB connection.
			LJUD.goOne(intHandle);
			
			//Get the temperature reading.
			LJUD.getResult(intHandle, LJUD.Constants.ioSHT_GET_READING, LJUD.Constants.chSHT_TEMP, refValue);
			dblValue = refValue.getValue();
			System.out.println("Temp Probe A = " + dblValue + " deg K");
			System.out.println("Temp Probe A = " + (dblValue - 273.15) + " deg C");
			System.out.println("Temp Probe A = " + (((dblValue - 273.15) * 1.8) + 32) + " deg F");
			
			//Get the humidity reading.
			LJUD.getResult(intHandle, LJUD.Constants.ioSHT_GET_READING, LJUD.Constants.chSHT_RH, refValue);
			System.out.println("RH Probe A = " + refValue.getValue() + " percent\n");
			
			//Set FIO7 to output-low to disable probe A. 
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_DIGITAL_BIT, 7, 0, 0);
			
			//Set DAC0 to 3.3 volts to enable probe B.
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_DAC, 0, 3.3, 0);
			
			//Since the DACs on the U3 are slower than the communication speed,
			//we put a delay here to make sure the DAC has time to rise to 3.3 volts
			//before communicating with the EI-1050.
			Thread.sleep(30);  //Wait 30 ms.
			
			//Now, an add/go/get block to get the temp  humidity at the same time.
			//Request a temperature reading from the EI-1050.
			LJUD.addRequest(intHandle, LJUD.Constants.ioSHT_GET_READING, LJUD.Constants.chSHT_TEMP, 0, 0, 0);
			
			//Request a humidity reading from the EI-1050.
			LJUD.addRequest(intHandle, LJUD.Constants.ioSHT_GET_READING, LJUD.Constants.chSHT_RH, 0, 0, 0);
			
			//Execute the requests.  Will take about 0.5 seconds with a USB high-high
			//or Ethernet connection, and about 1.5 seconds with a normal USB connection.
			LJUD.goOne(intHandle);
	
			//Get the temperature reading.
			LJUD.getResult(intHandle, LJUD.Constants.ioSHT_GET_READING, LJUD.Constants.chSHT_TEMP, refValue);
			dblValue = refValue.getValue();
			System.out.println("Temp Probe B = " + dblValue + " deg K");
			System.out.println("Temp Probe B = " + (dblValue - 273.15) + " deg C");
			System.out.println("Temp Probe B = " + (((dblValue - 273.15) * 1.8) + 32) + " deg F");
	
			//Get the humidity reading.
			LJUD.getResult(intHandle, LJUD.Constants.ioSHT_GET_READING, LJUD.Constants.chSHT_RH, refValue);
			System.out.println("RH Probe B = " + refValue.getValue() + " percent\n");
	
			//Set DAC0 to 0 volts to disable probe B.
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_DAC, 0, 0.0, 0);

			//If we were going to loop and talk to probe A next, we would
			//want a delay here to make sure the DAC falls to 0 volts
			//before enabling probe A.
			Thread.sleep(30);  //Wait 30 ms.
	
			//End of dual probe code.
			*/
		}
		catch(LJUDException le) {
			handleLJUDException(le);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		new EI1050().runExample();
	}

}