/**
 * Demonstrates SPI communication.
 * 
 * You can short MOSI to MISO for testing.
 * 
 * MOSI    FIO6
 * MISO    FIO7
 * CLK     FIO4
 * CS      FIO5
 * 
 * If you short MISO to MOSI, then you will read back the same bytes that you write.  If you short
 * MISO to GND, then you will read back zeros.  If you short MISO to VS or leave it
 * unconnected, you will read back 255s.
 *
 * support@labjack.com
 */

import com.sun.jna.*;
import com.sun.jna.ptr.*;
import com.labjack.LJUD;
import com.labjack.LJUDException;

public class SPI {

	public SPI() {
	}

	private void handleLJUDException(LJUDException e) {
		e.printStackTrace();
		if(e.getError() > LJUD.Errors.MIN_GROUP_ERROR.getValue()) {
			System.exit(-1);
		}
	}

	//Displays warning message if there is one. Error values < 0 are warnings
	//and do not cause a LJUDException in the LJUD class.
	private void checkForWarning(int error) {
		Pointer errorStringPtr = new Memory(256);
		if(error < 0) {
			LJUD.errorToString(error, errorStringPtr);
			System.out.println("Warning: " + errorStringPtr.getString(0).trim());
		}
	}
	
	public void runExample() {
		try {
			int intErrorcode = 0;
			IntByReference refIOType = new IntByReference(0);
			IntByReference refChannel = new IntByReference(0);
			DoubleByReference refValue = new DoubleByReference(0);
			IntByReference refHandle = new IntByReference(0);
			int intHandle = 0;
			DoubleByReference refNumSPIBytesToTransfer = new DoubleByReference(0);
			byte[] dataArray = new byte[50];
			DoubleByReference dummyDouble = new DoubleByReference(0.0);
			IntByReference dummyInt = new IntByReference(0);
	
			//Open the LabJack U3.
			intErrorcode = LJUD.openLabJack(LJUD.Constants.dtU3, LJUD.Constants.ctUSB, "1", 1, refHandle);
			checkForWarning(intErrorcode);
			intHandle = refHandle.getValue();
	
			//Start by using the pin_configuration_reset IOType so that all
			//pin assignments are in the factory default condition.
			LJUD.ePut(intHandle, LJUD.Constants.ioPIN_CONFIGURATION_RESET, 0, 0, 0);
	
			//First, configure the SPI communication.
	
			//Enable automatic chip-select control.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chSPI_AUTO_CS, 1, 0, 0);
	
			//Do not disable automatic digital i/o direction configuration.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chSPI_DISABLE_DIR_CONFIG, 0, 0, 0);
	
			//Mode A:  CPHA=0, CPOL=0.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chSPI_MODE, 0, 0, 0);
	
			//80kHz clock.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chSPI_CLOCK_FACTOR, 0, 0, 0);
	
			//MOSI is FIO6
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chSPI_MOSI_PIN_NUM, 6, 0, 0);
		
			//MISO is FIO7
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chSPI_MISO_PIN_NUM, 7, 0, 0);
	
			//CLK is FIO4
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chSPI_CLK_PIN_NUM, 4, 0, 0);
	
			//CS is FIO5
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chSPI_CS_PIN_NUM, 5, 0, 0);
	
			//Execute the requests on a single LabJack.
			LJUD.goOne(intHandle);
	
			//Get all the results just to check for errors.
			LJUD.getFirstResult(intHandle, refIOType, refChannel, refValue, dummyInt, dummyDouble);
			boolean isDone = false;
			while(!isDone) {
				try {
					LJUD.getNextResult(intHandle, refIOType, refChannel, refValue, dummyInt, dummyDouble);
				}
				catch(LJUDException le) {
					if(le.getError() == LJUD.Errors.NO_MORE_DATA_AVAILABLE.getValue()) {
						isDone = true;
					}
					else {
						throw le;
					}
				}
			}
	
			//This example transfers 4 test bytes.
			refNumSPIBytesToTransfer.setValue(4);
			dataArray[0] = (byte)170;
			dataArray[1] = (byte)240;
			dataArray[2] = (byte)170;
			dataArray[3] = (byte)240;
		
			//Transfer the data.  The write and read is done at the same time.
			LJUD.eGet(intHandle, LJUD.Constants.ioSPI_COMMUNICATION, 0, refNumSPIBytesToTransfer, dataArray);
	
			//Display the read data.
			for(int i = 0; i < 4; i++) {
				System.out.println("dataArray[" + i + "] = " + (dataArray[i]&0xFF));
			}
		}
		catch(LJUDException le) {
			handleLJUDException(le);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}


	public static void main(String[] args) {
		new SPI().runExample();
	}
}
