/**
 * Demonstrates using the TCVoltsToTemp to yield temperature readings for
 * a thermocouple connected to a U3 via a LJTick-InAmp
 *
 * Setup:
 * TC plus to INA+.
 * TC minus to INA-.
 * 10k resistor from INA- to GND on LJTIA (a short is also acceptable in most cases).
 * LJTIA connected to AIN0/AIN1 block on U3.
 * LJTIA offset set to 0.4 volts.
 * LJTIA channel A gain set to 51.
 *
 * For the best accuracy, we recommend doing a quick offset calibration as
 * described in the comments below, and using an external cold-junction
 * temperature sensor rather than the internal U3 temp sensor.  The internal
 * temp sensor is often sufficient, but an external sensor, such as 
 * the LM34CAZ (national.com) or the EI-1034 (labjack.com), placed near
 * the LJTIA, can provide better CJC, particularly when the U3 itself
 * is subjected to varying temperatures.
 *
 * support@labjack.com
 */

import java.io.*;
import java.text.*;
import com.sun.jna.*;
import com.sun.jna.ptr.*;
import com.labjack.LJUD;
import com.labjack.LJUDException;

public class Thermocouple {

	private Thermocouple() {
	}
	
	private void handleLJUDException(LJUDException e) {
		e.printStackTrace();
		if(e.getError() > LJUD.Errors.MIN_GROUP_ERROR.getValue()) {
			System.exit(-1);
		}
	}

	//Displays warning message if there is one. Error values < 0 are warnings
	//and do not cause a LJUDException in the LJUD class.
	private void checkForWarning(int error) {
		Pointer errorStringPtr = new Memory(256);
		if(error < 0) {
			LJUD.errorToString(error, errorStringPtr);
			System.out.println("Warning: " + errorStringPtr.getString(0).trim());
		}
	}

	public void runExample()
	{
		try {
			int intErrorcode = 0;
			IntByReference refHandle = new IntByReference(0);
			int intHandle = 0;
	
			IntByReference refIOType = new IntByReference(0);
			IntByReference refChannel = new IntByReference(0);
			DoubleByReference refValue = new DoubleByReference(0.0);
			DoubleByReference refValueAIN = new DoubleByReference(0.0);  //Analog Voltage Value
		
			int tempChannel = 0;  //Channel which the TC/LJTIA is on (AIN0/FIO0).
	
			double tcVolts = 0;
			DoubleByReference refCJTempK = new DoubleByReference(0.0);
			DoubleByReference refTCTempK = new DoubleByReference(0.0);
			int tcType = LJUD.Constants.ttK;
			//Set the temperature sensor to a k type thermocouple
			//Possible Thermocouple types are:
			//LJUD.Constants.ttB = 6001
			//LJUD.Constants.ttE = 6002
			//LJUD.Constants.ttJ = 6003
			//LJUD.Constants.ttK = 6004
			//LJUD.Constants.ttN = 6005
			//LJUD.Constants.ttR = 6006
			//LJUD.Constants.ttS = 6007
			//LJUD.Constants.ttT = 6008
	
			IntByReference dummyInt = new IntByReference(0);
			DoubleByReference dummyDouble = new DoubleByReference(0.0);
	
			BufferedReader br = new BufferedReader(
					new InputStreamReader(System.in));
			String line = "";
	
			//Offset calibration:  The nominal voltage offset of the LJTick is
			//0.4 volts.  For improved accuracy, though, you should measure the
			//overall system offset.  We know that if the end of the TC is at the
			//same temperature as the cold junction, the voltage should be zero.
			//Put the end of the TC near the LJTIA to make sure they are at the same
			//temperature, and note the voltage measured by FIO4.  This is the actual
			//offset that can be entered below.
			double offsetVoltage = 0.4;
	
			//Open the first found LabJack U3 via USB.
			intErrorcode = LJUD.openLabJack(LJUD.Constants.dtU3, LJUD.Constants.ctUSB, "1", 1, refHandle);
			checkForWarning(intErrorcode);
			intHandle = refHandle.getValue();
	
			//Reset all U3 ports to their default configuration.  
			//This is recommended prior to using the U3 to ensure the U3's ports are 
			//in the proper state.
			LJUD.ePut(intHandle, LJUD.Constants.ioPIN_CONFIGURATION_RESET, 0, 0, 0);
	
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_ANALOG_ENABLE_BIT, tempChannel, 1.0, 0);

			//Acquire temperature readings in a loop.
			while(true) {
				//Add analog input requests.
				LJUD.addRequest(intHandle, LJUD.Constants.ioGET_AIN, tempChannel, 0, 0, 0);
	
				//Add request for internal temperature reading -- Internal temp sensor uses 
				//analog input channel 30 on the U3.
				LJUD.addRequest(intHandle, LJUD.Constants.ioGET_AIN, 30, 0, 0, 0);
	
				//Execute all requests on the labjack intHandle.
				LJUD.goOne(intHandle);
	
				//Get all the results.  The first result should be the voltage reading of the 
				//temperature channel.
				LJUD.getFirstResult(intHandle, refIOType, refChannel, refValueAIN, dummyInt, dummyDouble);
	
				//The second result should be the internal temp sensor.
				LJUD.getNextResult(intHandle, refIOType, refChannel, refCJTempK, dummyInt, dummyDouble);
	
				//Display Voltage Reading
				System.out.println("Analog " + tempChannel + "           :" + refValueAIN.getValue());
	
				//Display the internal temperature sensor reading.  This example uses
				//that value for cold junction compensation.
				System.out.println("U3 internal sensor :" + refCJTempK.getValue() + " deg K");
	
				//To get the thermocouple voltage we subtract the offset from the AIN
				//voltage and divide by the LJTIA gain.
				tcVolts = (refValueAIN.getValue() - offsetVoltage)/51;
	
				//Convert TC voltage to temperature.
				LJUD.tcVoltsToTemp(tcType, tcVolts, refCJTempK.getValue(), refTCTempK);
	
				//Display Temperature
				System.out.println("Thermocouple sensor:" + refTCTempK.getValue() + " deg K\n");

				System.out.println("\nPress Enter to go again or (q) and Enter to quit");
				line = br.readLine().toUpperCase().trim();
				if(line.equals("Q")) {
					return;
				}
			}
		}
		catch(LJUDException le) {
			handleLJUDException(le);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public static void main(String[] args) {
		new Thermocouple().runExample();
	}
}