/**
 * Demonstrates asynchronous communication using a loopback from FIO2 to FIO3
 * on a LabJack U6.
 *
 * support@labjack.com
 */

import com.sun.jna.*;
import com.sun.jna.ptr.*;
import com.labjack.LJUD;
import com.labjack.LJUDException;

public class Asynch {
	
	public Asynch() {
	}
	
	private void handleLJUDException(LJUDException e) {
		e.printStackTrace();
		if(e.getError() > LJUD.Errors.MIN_GROUP_ERROR.getValue()) {
			System.exit(-1);
		}
	}

	//Displays warning message if there is one. Error values < 0 are warnings
	//and do not cause a LJUDException in the LJUD class.
	private void checkForWarning(int error) {
		Pointer errorStringPtr = new Memory(256);
		if(error < 0) {
			LJUD.errorToString(error, errorStringPtr);
			System.out.println("Warning: " + errorStringPtr.getString(0).trim());
		}
	}

	public void runExample() {
		try{
			int intErrorcode = 0;
			int intHandle = 0;
			IntByReference refHandle = new IntByReference(0);
			DoubleByReference refNumBytes = new DoubleByReference(0);
			byte[] array = new byte[256];
	
			//Open the LabJack.
			intErrorcode = LJUD.openLabJack(LJUD.Constants.dtU6, LJUD.Constants.ctUSB, "1", 1, refHandle);
			checkForWarning(intErrorcode);
			intHandle = refHandle.getValue();

			//48 MHz timer clock base.
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chTIMER_CLOCK_BASE, LJUD.Constants.tc48MHZ_DIV, 0);

			//Set clock divisor to 1, so timer clock is 48 MHz.
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chTIMER_CLOCK_DIVISOR, 1, 0);

			//Set timer/counter pin offset to 2. TX and RX appear after any timers and counters on U6.
			//We have no timers or counters enabled, so TX=FIO2 and RX=FIO3.
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chTIMER_COUNTER_PIN_OFFSET, 2, 0);

			//Set data rate for 9600 bps communication.
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chASYNCH_BAUDFACTOR, 63036, 0);

			//Enable UART.
			LJUD.ePut(intHandle, LJUD.Constants.ioASYNCH_COMMUNICATION, LJUD.Constants.chASYNCH_ENABLE, 1, 0);
	
			//Transmit 2 bytes.
			refNumBytes.setValue(2);
			array[0] = (byte)12;
			array[1] = (byte)20;
			LJUD.eGet(intHandle, LJUD.Constants.ioASYNCH_COMMUNICATION, LJUD.Constants.chASYNCH_TX, refNumBytes, array);

			//Read 2 bytes.
			refNumBytes.setValue(9999);  //Dummy values so we can see them change.
			array[0] = (byte)222;
			array[1] = (byte)222;
			LJUD.eGet(intHandle, LJUD.Constants.ioASYNCH_COMMUNICATION, LJUD.Constants.chASYNCH_RX, refNumBytes, array);
	
			//Display the read data.
			System.out.println("Pre-read buffer size = " + refNumBytes.getValue());
			System.out.print("Read data = ");
			for(int i = 0; i < refNumBytes.getValue(); i++) {
				System.out.print((array[i]&0xFF) + " ");
			}
			System.out.println("");
		}
		catch(LJUDException le) {
			handleLJUDException(le);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		new Asynch().runExample();
	}
}