/**
 * Basic command/response UD U6 example.
 * For testing timers, please attach a jumper to FIO0 and tap GND
 *
 * support@labjack.com
 */

import java.io.*;
import java.text.*;
import com.sun.jna.*;
import com.sun.jna.ptr.*;
import com.labjack.LJUD;
import com.labjack.LJUDException;

public class Simple {

	public Simple() {
	}

	private void handleLJUDException(LJUDException e) {
		e.printStackTrace();
		if(e.getError() > LJUD.Errors.MIN_GROUP_ERROR.getValue()) {
			System.exit(-1);
		}
	}

	//Displays warning message if there is one. Error values < 0 are warnings
	//and do not cause a LJUDException in the LJUD class.
	private void checkForWarning(int error) {
		Pointer errorStringPtr = new Memory(256);
		if(error < 0) {
			LJUD.errorToString(error, errorStringPtr);
			System.out.println("Warning: " + errorStringPtr.getString(0).trim());
		}
	}

	public void runExample() {
		try {
			int intErrorcode = 0;
			IntByReference refIOType = new IntByReference(0);
			IntByReference refChannel = new IntByReference(0);
			DoubleByReference refValue = new DoubleByReference(0);
			int intIOType = 0;
			int intChannel = 0;
			double dblValue = 0.0;
			double value2 = 9999, value3 = 9999;
			double valueDIBit = 9999, valueDIPort = 9999, valueCounter = 9999;
			int intHandle = 0;
			IntByReference refHandle = new IntByReference(0);
			IntByReference dummyInt = new IntByReference(0);
			DoubleByReference dummyDouble = new DoubleByReference(0.0);
			boolean isDone = false;
	
			NumberFormat formatter = new DecimalFormat("0.000");
			String line = "";
			BufferedReader br = new BufferedReader(
					new InputStreamReader(System.in));
	
			//Read and display the UD versions.
			System.out.println("UD Driver Version = " + formatter.format(LJUD.getDriverVersion()));
	
			//Open the first found LabJack U6.
			intErrorcode = LJUD.openLabJack(LJUD.Constants.dtU6, LJUD.Constants.ctUSB, "1", 1, refHandle);
			checkForWarning(intErrorcode);
			intHandle = refHandle.getValue();
	
			//First some configuration commands. These will be done with the ePut
			//function which combines the add/go/get into a single call.
	
			//Configure the analog input range on channels 2 and 3 for bipolar gain=1.
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_AIN_RANGE, 2, LJUD.Constants.rgBIP10V, 0);
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_AIN_RANGE, 3, LJUD.Constants.rgBIP10V, 0);
			
			//Enable Counter1 which will appear on FIO0 (assuming no other
			//program has enabled any timers or Counter0).
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_COUNTER_ENABLE, 1, 1, 0);

			//Now we add requests to write and read I/O.  These requests
			//will be processed repeatedly by go/get statements in every
			//iteration of the while loop below.
	
			//Request AIN2 and AIN3.
			LJUD.addRequest(intHandle, LJUD.Constants.ioGET_AIN, 2, 0, 0, 0);
			LJUD.addRequest(intHandle, LJUD.Constants.ioGET_AIN, 3, 0, 0, 0);
			
			//Set DAC0 to 2.5 volts.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_DAC, 0, 2.5, 0, 0);
			
			//Read digital input FIO1.
			LJUD.addRequest(intHandle, LJUD.Constants.ioGET_DIGITAL_BIT, 1, 0, 0, 0);

			//Set digital output FIO2 to output-high.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_DIGITAL_BIT, 2, 1, 0, 0);

			//Read digital inputs FIO3 through FIO7.
			LJUD.addRequest(intHandle, LJUD.Constants.ioGET_DIGITAL_PORT, 3, 0, 5, 0);
	
			//Request the value of Counter1.
			LJUD.addRequest(intHandle, LJUD.Constants.ioGET_COUNTER, 1, 0, 0, 0);
	
			while(true) {
				//Execute the requests.
				LJUD.goOne(intHandle);
				
				//Get all the results. The input measurement results are stored.
				//All other results are for configuration or output requests so 
				//we are just checking whether there was an error.
				LJUD.getFirstResult(intHandle, refIOType, refChannel, refValue, dummyInt, dummyDouble);
	
				isDone = false;
				try {
					while(!isDone) {
						intIOType = refIOType.getValue();
						intChannel = refChannel.getValue();
						dblValue = refValue.getValue();
						if(intIOType == LJUD.Constants.ioGET_AIN) {
							if(intChannel == 2) {
								value2 = dblValue;
							}
							if(intChannel == 3) {
								value3 = dblValue;
							}
						}
		
						if(intIOType == LJUD.Constants.ioGET_DIGITAL_BIT) {
							valueDIBit = dblValue;
						}
		
						if(intIOType == LJUD.Constants.ioGET_DIGITAL_PORT) {
							valueDIPort = dblValue;
						}
		
						if(intIOType == LJUD.Constants.ioGET_COUNTER) {
							valueCounter = dblValue;
						}
						
						LJUD.getNextResult(intHandle, refIOType, refChannel, refValue, dummyInt, dummyDouble);
					}
				}
				catch(LJUDException le) {
					if(le.getError() == LJUD.Errors.NO_MORE_DATA_AVAILABLE.getValue()) {
						isDone = true;
					}
					else {
						throw le;
					}
				}

				System.out.println("AIN2 = " + value2);
				System.out.println("AIN3 = " + value3);
				System.out.println("FIO1 = " + valueDIBit);
				System.out.println("FIO3-FIO7 = " + valueDIPort);	//Will read 31 if all 5 lines are pulled-high as normal.
				System.out.println("Counter1 (FIO0) " + valueCounter);
	
				System.out.println("\nPress Enter to go again or (q) and Enter to quit");
				line = br.readLine().toUpperCase().trim();
				if(line.equals("Q")) {
					return;
				}
			}
		}
		catch(LJUDException le) {
			handleLJUDException(le);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		new Simple().runExample();
	}
}