/**
 * Demonstrates using the TCVoltsToTemp function to yield temperature
 * readings for a thermocouple directly connected to a U6.
 *
 * Setup:
 * TC plus to AIN0.
 * TC minus to GND.
 *
 * If you do not want all the thermocouple minus leads tied together
 * at GND, you can use differential channels instead.  Connect the minus
 * to AIN1 in that case, and use a 10k resistor from AIN1 to GND to
 * provide a path for bias currents.
 *
 * support@labjack.com
 */

import java.io.*;
import java.text.*;
import com.sun.jna.*;
import com.sun.jna.ptr.*;
import com.labjack.LJUD;
import com.labjack.LJUDException;

public class Thermocouple {

	private Thermocouple() {
	}

	private void handleLJUDException(LJUDException e) {
		e.printStackTrace();
		if(e.getError() > LJUD.Errors.MIN_GROUP_ERROR.getValue()) {
			System.exit(-1);
		}
	}

	//Displays warning message if there is one. Error values < 0 are warnings
	//and do not cause a LJUDException in the LJUD class.
	private void checkForWarning(int error) {
		Pointer errorStringPtr = new Memory(256);
		if(error < 0) {
			LJUD.errorToString(error, errorStringPtr);
			System.out.println("Warning: " + errorStringPtr.getString(0).trim());
		}
	}

	public void runExample() {
		try {
			int intErrorcode = 0;
			IntByReference refHandle = new IntByReference(0);
			int intHandle = 0;
	
			IntByReference refIOType = new IntByReference(0);
			IntByReference refChannel = new IntByReference(0);
			DoubleByReference refValue = new DoubleByReference(0.0);
			DoubleByReference refValueAIN = new DoubleByReference(0.0);  //Analog Voltage Value
			double ainResolution = 0; //Configure resolution of the analog inputs. Setting to default (0 = index 8 or 9 for Pro).
			//See section 2.6 / 3.1 for more information.

			double range = LJUD.Constants.rgBIPP1V;
			int tempChannel = 0;  //Channel which the TC/LJTIA is on (AIN0).

			double tcVolts = 0.0;
			double cjTempK = 0.0;
			DoubleByReference refInternalTemp = new DoubleByReference(0.0);
			DoubleByReference refTCTempK = new DoubleByReference(0.0);
			int tcType = LJUD.Constants.ttK;
			//Set the temperature sensor to a k type thermocouple
			//Possible Thermocouple types are:
			//LJUD.Constants.ttB = 6001
			//LJUD.Constants.ttE = 6002
			//LJUD.Constants.ttJ = 6003
			//LJUD.Constants.ttK = 6004
			//LJUD.Constants.ttN = 6005
			//LJUD.Constants.ttR = 6006
			//LJUD.Constants.ttS = 6007
			//LJUD.Constants.ttT = 6008

			IntByReference dummyInt = new IntByReference(0);
			DoubleByReference dummyDouble = new DoubleByReference(0.0);

			BufferedReader br = new BufferedReader(
					new InputStreamReader(System.in));
			String line = "";
			
			//Open the first found LabJack U6 via USB.
			intErrorcode = LJUD.openLabJack(LJUD.Constants.dtU6, LJUD.Constants.ctUSB, "1", 1, refHandle);
			checkForWarning(intErrorcode);
			intHandle = refHandle.getValue();
	
			//Configure the desired resolution.  
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chAIN_RESOLUTION, ainResolution, 0);
	
			//Set the range on the analog input channel to +/-0.1 volts (x100 gain).
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_AIN_RANGE, tempChannel, range, 0);

			//Acquire temperature readings in a loop.
			while(true) {
				//Add analog input requests.
				LJUD.addRequest(intHandle, LJUD.Constants.ioGET_AIN, tempChannel, 0, 0, 0);
	
				//Add request for internal temperature reading -- Internal temp sensor uses 
				//analog input channel 14 on the U6.
				LJUD.addRequest(intHandle, LJUD.Constants.ioGET_AIN, 14, 0, 0, 0);
	
				//Execute all requests on the labjack intHandle.
				LJUD.goOne(intHandle);
	
				//Get all the results.  The first result should be the voltage reading of the 
				//temperature channel.
				LJUD.getFirstResult(intHandle, refIOType, refChannel, refValueAIN, dummyInt, dummyDouble);
	
				//The second result should be the internal temp sensor.
				LJUD.getNextResult(intHandle, refIOType, refChannel, refInternalTemp, dummyInt, dummyDouble);

				//The cold junction is the screw-terminal block where the thermocouple
				//is connected.  As discussed in the U6 User's Guide, add 2.5 degrees C
				//to the internal temp sensor reading.  If using the CB37 rather than
				//the built-in screw terminals, just add 1.0 degrees C.
				cjTempK = refInternalTemp.getValue() + 2.5;
	
				//Display Voltage Reading
				System.out.println("Analog " + tempChannel + "           :" + refValueAIN.getValue());
	
				//Display the internal temperature sensor reading.  This example uses
				//that value for cold junction compensation.
				System.out.println("U6 internal sensor :" + refInternalTemp.getValue() + " deg K");
	
				//Convert TC voltage to temperature.
				LJUD.tcVoltsToTemp(tcType, refValueAIN.getValue(), cjTempK, refTCTempK);
	
				//Display Temperature
				System.out.println("Thermocouple sensor: " + refTCTempK.getValue() + " deg K");
	
				System.out.println("\nPress Enter to go again or (q) and Enter to quit");
				line = br.readLine().toUpperCase().trim();
				if(line.equals("Q")) {
					return;
				}
			}
		
		}
		catch(LJUDException le) {
			handleLJUDException(le);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public static void main(String[] args) {
		new Thermocouple().runExample();
	}
}
