/**
 * Basic U6 example does a PWM output and a counter input, using AddGoGet method.
 *
 * support@labjack.com
 */

import com.sun.jna.*;
import com.sun.jna.ptr.*;
import com.labjack.LJUD;
import com.labjack.LJUDException;

public class TimerCounter {

	public TimerCounter() {
	}

	private void handleLJUDException(LJUDException e) {
		e.printStackTrace();
		if(e.getError() > LJUD.Errors.MIN_GROUP_ERROR.getValue()) {
			System.exit(-1);
		}
	}

	//Displays warning message if there is one. Error values < 0 are warnings
	//and do not cause a LJUDException in the LJUD class.
	private void checkForWarning(int error) {
		Pointer errorStringPtr = new Memory(256);
		if(error < 0) {
			LJUD.errorToString(error, errorStringPtr);
			System.out.println("Warning: " + errorStringPtr.getString(0).trim());
		}
	}

	public void runExample() {
		try {
			int intErrorcode;
			IntByReference refIOType = new IntByReference(0);
			IntByReference refChannel = new IntByReference(0);
			DoubleByReference refValue = new DoubleByReference(0);
			IntByReference refHandle = new IntByReference(0);
			int intHandle = 0;
			DoubleByReference dummyDouble = new DoubleByReference(0.0);
			IntByReference dummyInt = new IntByReference(0);

			//Open the first found LabJack U6.
			intErrorcode = LJUD.openLabJack(LJUD.Constants.dtU6, LJUD.Constants.ctUSB, "1", 1, refHandle);
			checkForWarning(intErrorcode);
			intHandle = refHandle.getValue();

			//First requests to configure the timer and counter.  These will be
			//done with and add/go/get block.

			//Set the timer/counter pin offset to 0, which will put the first
			//timer/counter on FIO0.
			LJUD.addRequest(intHandle,  LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chTIMER_COUNTER_PIN_OFFSET, 0, 0, 0);

			//Use the 48 MHz timer clock base with divider.  Since we are using clock with divisor
			//support, Counter0 is not available.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chTIMER_CLOCK_BASE, LJUD.Constants.tc48MHZ_DIV, 0, 0);

			//Set the divisor to 48 so the actual timer clock is 1 MHz.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chTIMER_CLOCK_DIVISOR, 48, 0, 0);

			//Enable 1 timer.  It will use FIO0.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chNUMBER_TIMERS_ENABLED, 1, 0, 0);

			//Configure Timer0 as 8-bit PWM.  Frequency will be 1M/256 = 3906 Hz.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_TIMER_MODE, 0, LJUD.Constants.tmPWM8, 0, 0);

			//Set the PWM duty cycle to 50%.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_TIMER_VALUE, 0, 32768, 0, 0);

			//Enable Counter1.  It will use FIO1 since 1 timer is enabled.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_COUNTER_ENABLE, 1, 1, 0, 0);

			//Execute the requests.
			LJUD.goOne(intHandle);

			//Get all the results just to check for errors.
			LJUD.getFirstResult(intHandle, refIOType, refChannel, refValue, dummyInt, dummyDouble);
			boolean isDone = false;
			while(!isDone) {
				try {
					LJUD.getNextResult(intHandle, refIOType, refChannel, refValue, dummyInt, dummyDouble);
				}
				catch(LJUDException le) {
					if(le.getError() == LJUD.Errors.NO_MORE_DATA_AVAILABLE.getValue()) {
						isDone = true;
					}
					else {
						throw le;
					}
				}
			}

			//Wait 1 second.
			Thread.sleep(1000);

			//Request a read from the counter.
			LJUD.eGet(intHandle, LJUD.Constants.ioGET_COUNTER, 1, refValue, 0);

			//This should read roughly 4k counts if FIO0 is shorted to FIO1.
			System.out.println("Counter = " + refValue.getValue());

			//Wait 1 second.
			Thread.sleep(1000);

			//Request a read from the counter.
			LJUD.eGet(intHandle, LJUD.Constants.ioGET_COUNTER, 1, refValue, 0);

			//This should read about 3906 counts more than the previous read.
			System.out.println("Counter = " + refValue.getValue());

			//Reset all pin assignments to factory default condition.
			LJUD.ePut(intHandle, LJUD.Constants.ioPIN_CONFIGURATION_RESET, 0, 0, 0);

			//The PWM output sets FIO0 to output, so we do a read here to set
			//it to input.
			LJUD.eGet(intHandle, LJUD.Constants.ioGET_DIGITAL_BIT, 0, refValue, 0);
		}
		catch(LJUDException le) {
			handleLJUDException(le);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public static void main(String[] args) {
		new TimerCounter().runExample();
	}
}