/**
 * Demonstrates using the add/go/get method to efficiently write and read
 * virtually all analog and digital I/O on the LabJack UE9.
 * Records the time for 1000 iterations and divides by 1000, to allow
 * verification of the basic command/response communication times of the
 * LabJack UE9 as documented in Section 3.1 of the UE9 User's Guide.
 *
 * support@labjack.com
 */

import java.util.Arrays;
import com.sun.jna.*;
import com.sun.jna.ptr.*;
import com.labjack.LJUD;
import com.labjack.LJUDException;

public class AllIO {

	public AllIO() {
	}
	
	private void handleLJUDException(LJUDException e) {
		e.printStackTrace();
		if(e.getError() > LJUD.Errors.MIN_GROUP_ERROR.getValue()) {
			System.exit(-1);
		}
	}

	//Displays warning message if there is one. Error values < 0 are warnings
	//and do not cause a LJUDException in the LJUD class.
	private void checkForWarning(int error) {
		Pointer errorStringPtr = new Memory(256);
		if(error < 0) {
			LJUD.errorToString(error, errorStringPtr);
			System.out.println("Warning: " + errorStringPtr.getString(0).trim());
		}
	}
	
	public void runExample() {
		try {
			int intErrorcode = 0;
			int intHandle = 0;
			IntByReference refHandle = new IntByReference(0);
			IntByReference refIOType = new IntByReference(0);
			IntByReference refChannel = new IntByReference(0);
			DoubleByReference refValue = new DoubleByReference(0);
			int intIOType = 0;
			int intChannel = 0;
			double dblValue = 0.0;
			
			long time = 0;
			int numIterations = 1000;
			final int numChannels = 5;  //Number of AIN channels, 0-16.
			int ainResolution = 12;

			double valueDIPort = 0;
			double[] valueAIN = new double[numChannels];
			Arrays.fill(valueAIN, 9999.0);
			
			IntByReference dummyInt = new IntByReference(0);
			DoubleByReference dummyDouble = new DoubleByReference(0.0);

			boolean isDone = false;
			
			//Open the first found LabJack UE9.
			intErrorcode = LJUD.openLabJack(LJUD.Constants.dtUE9, LJUD.Constants.ctUSB, "1", 1, refHandle);
			//intErrorcode = LJUD.openLabJack(LJUD.Constants.dtUE9, LJUD.Constants.ctETHERNET, "192.168.1.209", 0, refHandle);
			checkForWarning(intErrorcode);
			intHandle = refHandle.getValue();

			//Set DAC0 to 2.5 volts.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_DAC, 0, 2.5, 0, 0);

			//Set DAC1 to 3.5 volts.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_DAC, 1, 3.5, 0, 0);

			//Write all digital I/O.  Doing a bunch of bit instructions, rather than
			//the following port instruction, should not make a noticable difference
			//in the overall execution time.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_DIGITAL_PORT, 0, 0, 23, 0);

			//Configure the desired resolution.  Note that depending on resolution and
			//number of analog inputs, numIterations might need to be reduced from the
			//default above so the program does not take too long to execute.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chAIN_RESOLUTION, ainResolution, 0, 0);

			//Add analog input requests.
			for(int j = 0; j < numChannels; j++) {
				LJUD.addRequest(intHandle, LJUD.Constants.ioGET_AIN, j, 0, 0, 0);
			}

			//Request a read of all digital I/O.  Doing a bunch of bit instructions,
			//rather than the following port instruction, should not make a noticable
			//difference in the overall execution time.
			LJUD.addRequest(intHandle, LJUD.Constants.ioGET_DIGITAL_PORT, 0, 0, 23, 0);
	
			time = System.currentTimeMillis();
	
			for(int i = 0; i < numIterations; i++) {
				//Execute the requests.
				LJUD.goOne(intHandle);

				//Get all the results.  The input measurement results are stored.  All other
				//results are for configuration or output requests so we are just checking
				//whether there was an error.
				LJUD.getFirstResult(intHandle, refIOType, refChannel, refValue, dummyInt, dummyDouble);
				isDone = false;
				while(!isDone) {
					try {
						intIOType = refIOType.getValue();
						intChannel = refChannel.getValue();
						dblValue = refValue.getValue();
						if (intIOType == LJUD.Constants.ioGET_AIN) {
							valueAIN[intChannel] = dblValue;
						}
						else if (intIOType == LJUD.Constants.ioGET_DIGITAL_PORT) {
							valueDIPort = dblValue;
						}
						LJUD.getNextResult(intHandle, refIOType, refChannel, refValue, dummyInt, dummyDouble);
					}
					catch(LJUDException le) {
						if(le.getError() == LJUD.Errors.NO_MORE_DATA_AVAILABLE.getValue()) {
							isDone = true;
						}
						else {
							throw le;
						}
					}	
				}
			}

			time = System.currentTimeMillis() - time;

			System.out.println("Milliseconds per iteration = " + (double)time / (double)numIterations);
			System.out.println("\nDigital Input = " + valueDIPort);
			System.out.println("\nAIN readings from last iteration:\n");
			for(int j = 0; j < numChannels; j++) {
				System.out.println(valueAIN[j]);
			}
		}
		catch(LJUDException le) {
			handleLJUDException(le);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
    }

	public static void main(String[] args) {
		new AllIO().runExample();
	}
}
