/**
 * Demonstrates asynchronous communication using a loopback from PIN2 to 
 * PIN20 (TX0 to RX0) pins on the DB37 connector on a LabJack UE9.
 *
 * support@labjack.com
 */

import com.sun.jna.*;
import com.sun.jna.ptr.*;
import com.labjack.LJUD;
import com.labjack.LJUDException;

public class Asynch {
	
	public Asynch() {
	}
	
	private void handleLJUDException(LJUDException e) {
		e.printStackTrace();
		if(e.getError() > LJUD.Errors.MIN_GROUP_ERROR.getValue()) {
			System.exit(-1);
		}
	}

	//Displays warning message if there is one. Error values < 0 are warnings
	//and do not cause a LJUDException in the LJUD class.
	private void checkForWarning(int error) {
		Pointer errorStringPtr = new Memory(256);
		if(error < 0) {
			LJUD.errorToString(error, errorStringPtr);
			System.out.println("Warning: " + errorStringPtr.getString(0).trim());
		}
	}

	public void runExample() {
		try{
			int intErrorcode = 0;
			int intHandle = 0;
			IntByReference refHandle = new IntByReference(0);
			DoubleByReference refNumBytes = new DoubleByReference(0);
			byte[] array = new byte[256];
	
			//Open the LabJack.
			intErrorcode = LJUD.openLabJack(LJUD.Constants.dtUE9, LJUD.Constants.ctUSB, "1", 1, refHandle);
			//intErrorcode = LJUD.openLabJack(LJUD.Constants.dtUE9, LJUD.Constants.ctETHERNET, "192.168.1.209", 0, refHandle);
			checkForWarning(intErrorcode);
			intHandle = refHandle.getValue();

			//48 MHz timer clock base.
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chTIMER_CLOCK_BASE, LJUD.Constants.tcSYS, 0);

			//Set clock divisor to 1, so timer clock is 48 MHz.
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chTIMER_CLOCK_DIVISOR, 1, 0);

			//Set data rate for 9615 bps (compatible with 9600 bps).
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chASYNCH_BAUDFACTOR, 65224, 0);

			//Enable UART.
			LJUD.ePut(intHandle, LJUD.Constants.ioASYNCH_COMMUNICATION, LJUD.Constants.chASYNCH_ENABLE, 1, 0);

			//TX/RX appear on PIN2/PIN20 (TX0/RX0) pins on the DB37 connector.
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chTIMER_COUNTER_PIN_OFFSET, 2, 0);
			
			//Transmit 2 bytes.
			refNumBytes.setValue(2);
			
			array[0] = (byte)12;
			array[1] = (byte)20;
			LJUD.eGet(intHandle, LJUD.Constants.ioASYNCH_COMMUNICATION, LJUD.Constants.chASYNCH_TX, refNumBytes, array);

			//Read 2 bytes.
			refNumBytes.setValue(9999);  //Dummy values so we can see them change.
			array[0] = (byte)222;
			array[1] = (byte)222;
			LJUD.eGet(intHandle, LJUD.Constants.ioASYNCH_COMMUNICATION, LJUD.Constants.chASYNCH_RX, refNumBytes, array);
	
			//Display the read data.
			System.out.println("Pre-read buffer size = " + refNumBytes.getValue());
			System.out.print("Read data = ");
			for(int i = 0; i < refNumBytes.getValue(); i++) {
				System.out.print((array[i]&0xFF) + " ");
			}
			System.out.println("");
		}
		catch(LJUDException le) {
			handleLJUDException(le);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		new Asynch().runExample();
	}
}
