/**
 * Basic UE9 example does a PWM output and a counter input, using AddGoGet method.
 *
 * support@labjack.com
 */

import com.sun.jna.*;
import com.sun.jna.ptr.*;
import com.labjack.LJUD;
import com.labjack.LJUDException;

public class TimerCounter {

	public TimerCounter() {
	}

	private void handleLJUDException(LJUDException e) {
		e.printStackTrace();
		if(e.getError() > LJUD.Errors.MIN_GROUP_ERROR.getValue()) {
			System.exit(-1);
		}
	}

	//Displays warning message if there is one. Error values < 0 are warnings
	//and do not cause a LJUDException in the LJUD class.
	private void checkForWarning(int error) {
		Pointer errorStringPtr = new Memory(256);
		if(error < 0) {
			LJUD.errorToString(error, errorStringPtr);
			System.out.println("Warning: " + errorStringPtr.getString(0).trim());
		}
	}

	public void runExample() {
		try {
			int intErrorcode = 0;
			IntByReference refIOType = new IntByReference(0);
			IntByReference refChannel = new IntByReference(0);
			DoubleByReference refValue = new DoubleByReference(0);
			IntByReference refHandle = new IntByReference(0);
			int intHandle = 0;
			int resetCounter = 1;
			double count = -1;
			double highTime, lowTime;
			double period16 = 0, period32 = 0;

			DoubleByReference dummyDouble = new DoubleByReference(0.0);
			IntByReference dummyInt = new IntByReference(0);
			boolean isDone = false;
			
			//Open the LabJack UE9.
			intErrorcode = LJUD.openLabJack(LJUD.Constants.dtUE9, LJUD.Constants.ctUSB, "1", 1, refHandle);
			//intErrorcode = LJUD.openLabJack(LJUD.Constants.dtUE9, LJUD.Constants.ctETHERNET, "192.168.1.209", 0, refHandle);
			checkForWarning(intErrorcode);
			intHandle = refHandle.getValue();
	
			//Disable all timers and counters to put everything in a known initial state.
			//Disable the timer and counter, and the FIO lines will return to digital I/O.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chNUMBER_TIMERS_ENABLED, 0, 0, 0);
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_COUNTER_ENABLE, 0, 0, 0, 0);
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_COUNTER_ENABLE, 1, 0, 0, 0);
			LJUD.goOne(intHandle);
	
			//First we will output a square wave and count the number of pulses for about 1 second.
			//Connect a jumper on the UE9 from FIO0 (PWM output) to FIO1 (Counter0 input).
	
			//Use the fixed 750kHz timer clock source.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chTIMER_CLOCK_BASE, LJUD.Constants.tc750KHZ, 0, 0);

			//Set the divisor to 3 so the actual timer clock is 250kHz.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chTIMER_CLOCK_DIVISOR, 3, 0, 0);

			//Enable 1 timer.  It will use FIO0.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chNUMBER_TIMERS_ENABLED, 1, 0, 0);

			//Configure Timer0 as 8-bit PWM.  Frequency will be 250k/256 = 977 Hz.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_TIMER_MODE, 0, LJUD.Constants.tmPWM8, 0, 0);

			//Set the PWM duty cycle to 50%.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_TIMER_VALUE, 0, 32768, 0, 0);

			//Enable Counter0.  It will use FIO1 since 1 timer is enabled.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_COUNTER_ENABLE, 0, 1, 0, 0);

			//Execute the requests on a single LabJack.  The driver will use a
			//single low-level TimerCounter command to handle all the requests above.
			LJUD.goOne(intHandle);
	
			//Get all the results just to check for errors.
			isDone = false;
			while(!isDone) {
				try {
					LJUD.getNextResult(intHandle, refIOType, refChannel, refValue, dummyInt, dummyDouble);
				}
				catch(LJUDException le) {
					if(le.getError() == LJUD.Errors.NO_MORE_DATA_AVAILABLE.getValue()) {
						isDone = true;
					}
					else {
						throw le;
					}
				}
			}

			//Wait 1 second.
			Thread.sleep(1000);

			//Request a read from the counter.
			LJUD.eGet(intHandle, LJUD.Constants.ioGET_COUNTER, 0, refValue, 0);

			//This should read roughly 977 counts.
			System.out.println("Counter = " + refValue.getValue() + "\n");

			//Disable the timer and counter, and the FIO lines will return to digital I/O.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chNUMBER_TIMERS_ENABLED, 0, 0, 0);
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_COUNTER_ENABLE, 0, 0, 0, 0);
			LJUD.goOne(intHandle);

			//Output a square wave and measure the period.
			//Connect a jumper on the UE9 from FIO0 (PWM8 output) to
			//FIO1 (RISINGEDGES32 input) and FIO2 (RISINGEDGES16).

			//Use the fixed 750kHz timer clock source.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chTIMER_CLOCK_BASE, LJUD.Constants.tc750KHZ, 0, 0);

			//Set the divisor to 3 so the actual timer clock is 250kHz.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chTIMER_CLOCK_DIVISOR, 3, 0, 0);

			//Enable 3 timers.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chNUMBER_TIMERS_ENABLED, 3, 0, 0);

			//Configure Timer0 as 8-bit PWM.  Frequency will be 250k/256 = 977 Hz.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_TIMER_MODE, 0, LJUD.Constants.tmPWM8, 0, 0);

			//Set the PWM duty cycle to 50%.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_TIMER_VALUE, 0, 32768, 0, 0);

			//Configure Timer1 as 32-bit period measurement.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_TIMER_MODE, 1, LJUD.Constants.tmRISINGEDGES32, 0, 0);

			//Configure Timer2 as 16-bit period measurement.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_TIMER_MODE, 2, LJUD.Constants.tmRISINGEDGES16, 0, 0);

			//Execute the requests on a single LabJack.  The driver will use a
			//single low-level TimerCounter command to handle all the requests above.
			LJUD.goOne(intHandle);

			//Get all the results just to check for errors.
			isDone = false;
			while(!isDone) {
				try {
					LJUD.getNextResult(intHandle, refIOType, refChannel, refValue, dummyInt, dummyDouble);
				}
				catch(LJUDException le) {
					if(le.getError() == LJUD.Errors.NO_MORE_DATA_AVAILABLE.getValue()) {
						isDone = true;
					}
					else {
						throw le;
					}
				}
			}
	
			//Wait 1 second.
			Thread.sleep(1000);
	
			//Now read the period measurements from the 2 timers.  We
			//will use the Add/Go/Get method so that both
			//reads are done in a single low-level call.
	
			//Request a read from Timer1
			LJUD.addRequest(intHandle, LJUD.Constants.ioGET_TIMER, 1, 0, 0, 0);
	
			//Request a read from Timer2
			LJUD.addRequest(intHandle, LJUD.Constants.ioGET_TIMER, 2, 0, 0, 0);
	
			//Execute the requests on a single LabJack.  The driver will use a
			//single low-level TimerCounter command to handle all the requests above.
			LJUD.goOne(intHandle);
	
			//Get the results of the two read requests.
			LJUD.getFirstResult(intHandle, refIOType, refChannel, refValue, dummyInt, dummyDouble);
			isDone = false;
			try {
				while(!isDone) {
					if(refIOType.getValue() == LJUD.Constants.ioGET_TIMER) {
						switch(refChannel.getValue()) {
							case 1:
								period32 = refValue.getValue();
								break;
							case 2:
								period16 = refValue.getValue();
								break;
						}
					}
					LJUD.getNextResult(intHandle, refIOType, refChannel, refValue, dummyInt, dummyDouble);
				}
			}
			catch(LJUDException le) {
				if(le.getError() == LJUD.Errors.NO_MORE_DATA_AVAILABLE.getValue()) {
					isDone = true;
				}
				else {
					throw le;
				}
			}
			
			//Both period measurements should read about 256.  The timer
			//clock was set to 250 kHz, so each tick equals 4 microseconds, so
			//256 ticks means a period of 1024 microseconds which is a frequency
			//of 977 Hz.
			System.out.println("Period32 = " + period32);
			System.out.println("Period16 = " + period16 + "\n");
	
			//Disable the timer and counter, and the FIO lines will return to digital I/O.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chNUMBER_TIMERS_ENABLED, 0, 0, 0);
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_COUNTER_ENABLE, 0, 0, 0, 0);
			LJUD.goOne(intHandle);
	
			//Now we will output a 25% duty-cycle PWM output on Timer0 (FIO0) and measure
			//the duty cycle on Timer1 FIO1.  Requires Control firmware V1.21 or higher.
	
			//Use the fixed 750kHz timer clock source.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chTIMER_CLOCK_BASE, LJUD.Constants.tc750KHZ, 0, 0);

			//Set the divisor to 3 so the actual timer clock is 250kHz.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chTIMER_CLOCK_DIVISOR, 3, 0, 0);

			//Enable 2 timers.  They will use FIO0 and FIO1.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chNUMBER_TIMERS_ENABLED, 2, 0, 0);

			//Configure Timer0 as 8-bit PWM.  Frequency will be 250k/256 = 977 Hz.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_TIMER_MODE, 0, LJUD.Constants.tmPWM8, 0, 0);

			//Set the PWM duty cycle to 25%.  The passed value is the low time.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_TIMER_VALUE, 0, 49152, 0, 0);

			//Configure Timer1 as duty cycle measurement.
			LJUD.addRequest(intHandle, LJUD.Constants.ioPUT_TIMER_MODE, 1, LJUD.Constants.tmDUTYCYCLE, 0, 0);

			//Execute the requests on a single LabJack.  The driver will use a
			//single low-level TimerCounter command to handle all the requests above.
			LJUD.goOne(intHandle);

			//Get all the results just to check for errors.
			isDone = false;
			while(!isDone) {
				try {
					LJUD.getNextResult(intHandle, refIOType, refChannel, refValue, dummyInt, dummyDouble);
				}
				catch(LJUDException le) {
					if(le.getError() == LJUD.Errors.NO_MORE_DATA_AVAILABLE.getValue()) {
						isDone = true;
					}
					else {
						throw le;
					}
				}
			}
	
			//Wait a little so we are sure a duty cycle measurement has occured.
			Thread.sleep(100);
	
			//Request a read from Timer1.
			LJUD.eGet(intHandle, LJUD.Constants.ioGET_TIMER, 1, refValue, 0);
	
			//High time is LSW
			highTime = (double)(refValue.getValue() % (65536));
			//Low time is MSW
			lowTime = (double)(refValue.getValue() / (65536));
	
			System.out.println("High clicks = " + highTime);
			System.out.println("Low clicks = " + lowTime);
			System.out.println("Duty cycle = " + (100*highTime/(highTime+lowTime)));
	
			//Disable the timers, and the FIO lines will return to digital I/O.
			LJUD.ePut(intHandle, LJUD.Constants.ioPUT_CONFIG, LJUD.Constants.chNUMBER_TIMERS_ENABLED, 0, 0);
	
			//The PWM output sets FIO0 to output, so we do a read here to set
			//FIO0 to input.
			LJUD.eGet(intHandle, LJUD.Constants.ioGET_DIGITAL_BIT, 0, refValue, 0);
		}
		catch(LJUDException le) {
			handleLJUDException(le);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public static void main(String[] args) {
		new TimerCounter().runExample();
	}
}