package com.labjack;

import com.sun.jna.*;
import com.sun.jna.win32.StdCallLibrary;
import com.sun.jna.ptr.*;

/**
 * LJUD is the wrapper class to the LJUD driver's functions and constants.
 * Refer to the LabJackUD.h header file (in the "\Program Files\LabJack\Drivers" or
 * "\Program Files (x86)\LabJack\Drivers" folder) or online User's Guide for
 * functions and constants documentation:<br>
 * <br>
 * &nbsp;&nbsp;&nbsp;&nbsp;UE9: <a href="http://labjack.com/support/ue9/users-guide/4">http://labjack.com/support/ue9/users-guide/4</a><br>
 * &nbsp;&nbsp;&nbsp;&nbsp;U3: <a href="http://labjack.com/support/u3/users-guide/4">http://labjack.com/support/u3/users-guide/4</a><br>
 * &nbsp;&nbsp;&nbsp;&nbsp;U6: <a href="http://labjack.com/support/u6/users-guide/4">http://labjack.com/support/u6/users-guide/4</a><br>
 * <p>LJUD library C to Java differences:
 * <ul>
 * <li>C functions are implemented in the LJUD class as static methods. The
 * function name's first letter (and possibly more) have been changed to
 * lowercase for Java convention.
 * <li>C constants can be found in the {@link LJUD.Constants} class. The constant
 * name's "LJ_" prefix have been removed.
 * <li>C error code constants can be found in the {@link LJUD.Errors} enum. The
 * constants name's "LJE_" prefix have been removed.
 * <li>C function parameter names have the first letter (and possibly more)
 * changed to lowercase for Java convention.
 * <li>If the wrapper method detects an error it will throw a
 * {@link LJUDException} exception, setting the error value and exception
 * message. The following methods do not throw a LJUDException:
 * <ul>
 * <li>{@link #close() close}
 * <li>{@link #eAddGoGet(int, int, int[], int[], double[], int[], int[], IntByReference, int[]) eAddGoGet}
 * <li>{@link #getNextError(int, IntByReference, IntByReference) getNextError} 
 * <li>{@link #getStreamError(int) getStreamError}
 * <li>{@link #stringToConstant(String) stringToConstant}
 * <li>{@link #errorToString(int, Pointer) errorToString}
 * <li>{@link #getDriverVersion() getDriverVersion}
 * <li>{@link #getThreadID() getThreadID}
 * </ul>
 * <li>C parameters that are pass by reference are implemented in Java as
 * arrays, or JNA classes IntByReference or DoubleByReference for single value
 * references.
 * <li>C void * parameters are implemented in Java as a JNA class Pointer.
 * <li>C string parameters are implemented in Java as a String and pass by
 * reference strings are a JNA class Pointer.
 * <li>When using a JNA Pointer for C strings, the JNA Memory class needs to be used
 * to construct the pointer and allocate memory. For example:
 * <pre>
 * 
 * //Allocate 100 bytes of memory for a a 100 character C string.
 * Pointer stringPtr = new Memory(100);</pre>
 * <li>If you want to use AddRequest to pass an array to x1, use AddRequestPtr.
 * AddRequest methods are not being overloaded to accept arrays since they are
 * not 64-bit pointer safe for x1 and to allow users to handle their x1
 * memory to prevent it from going out of scope between the
 * AddRequest/Go/GetResult calls. Here's a quick example on passing a byte
 * array to AddRequestPtr:
 * <pre>
 * 
 * byte x1[] = new byte[] { 0x05, (byte)0xF8, 0x02, 0x0A, 0x00, 0x00,
 *         0x00, 0x00, 0x00, 0x00 };
 * Pointer px1 = new Memory(x1.length);
 * 
 * //Copy x1 byte array into px1 pointer's memory
 * px1.write(0, x1, 0, x1.length);
 * LJUD.AddRequestPtr(handle, LJUD.Constants.ioRAW_OUT, 0, x1.length,
 *         px1, 0);</pre>
 * </ul>
 * <p>Version History
 * <ul>
 * <li>1.00 - Initial release tested with LJUD driver v3.46. Windows support only.
 * </ul>
 * @author LabJack Corporation <support@labjack.com>
 * @version 1.00
 */
public class LJUD {
	public interface LabJackUD extends StdCallLibrary {
		LabJackUD INSTANCE = (LabJackUD)Native.loadLibrary("LabJackUD",
				LabJackUD.class);
	
		void Close();
		int ListAll(int DeviceType, int ConnectionType,
				IntByReference pNumFound, int[] pSerialNumbers, int[] pIDs,
				double[] pAddresses);
		int ListAllS(final String pDeviceType, final String pConnectionType,
				IntByReference pNumFound, int[] pSerialNumbers, int[] pIDs,
				double[] pAddresses);
		int OpenLabJack(int DeviceType, int ConnectionType,
				final String pAddress, int FirstFound, IntByReference pHandle);
		int OpenLabJackS(final String pDeviceType,
				final String pConnectionType, final String pAddress,
				int FirstFound, IntByReference pHandle);
		int AddRequest(int Handle, int IOType, int Channel, double Value,
				int x1, double UserData);
		int AddRequestS(int Handle, final String pIOType, int Channel,
				double Value, int x1, double UserData);
		int AddRequestSS(int Handle, final String IOType,
				final String pChannel, double Value, int x1, double UserData);
		int AddRequestPtr(int Handle, int IOType, int Channel, double Value,
				Pointer x1, double UserData);
		int Go();
		int GoOne(int Handle);
		int eGet(int Handle, int IOType, int Channel, DoubleByReference pValue,
				int x1);
		int eGetPtr(int Handle, int IOType, int Channel,
				DoubleByReference pValue, Pointer x1);
		int eGetS(int Handle, final String pIOType, int Channel,
				DoubleByReference pValue, int x1);
		int eGetSS(int Handle, final String pIOType, final String pChannel,
				DoubleByReference pValue, int x1);
		int ePut(int Handle, int IOType, int Channel, double Value, int x1);
		int ePutS(int Handle, final String pIOType, int Channel, double Value,
				int x1);
		int ePutSS(int Handle, final String pIOType, final String pChannel,
				double Value, int x1);
		int eGet_DblArray(int Handle, int IOType, int Channel,
				DoubleByReference pValue, double[] x1);
		int eGet_U8Array(int Handle, int IOType, int Channel,
				DoubleByReference pValue, byte[] x1); //unsigned char
		int eGetS_DblArray(int Handle, final String pIOType, int Channel,
				DoubleByReference pValue, double[] x1);
		int eGetS_U8Array(int Handle, final String pIOType, int Channel,
				DoubleByReference pValue, byte[] x1); //unsigned char
		int eGetSS_DblArray(int Handle, final String pIOType,
				final String pChannel, DoubleByReference pValue, double[] x1);
		int eGetSS_U8Array(int Handle, final String pIOType,
				final String pChannel, DoubleByReference pValue, byte[] x1); //unsigned char
		int GetResult(int Handle, int IOType, int Channel,
				DoubleByReference pValue);
		int GetResultS(int Handle, final String pIOType, int Channel,
				DoubleByReference pValue);
		int GetResultSS(int Handle, final String pIOType,
				final String pChannel, DoubleByReference pValue);
		int GetFirstResult(int Handle, IntByReference pIOType,
				IntByReference pChannel, DoubleByReference pValue,
				IntByReference px1, DoubleByReference pUserData);
		int GetNextResult(int Handle, IntByReference pIOType,
				IntByReference pChannel, DoubleByReference pValue,
				IntByReference px1, DoubleByReference pUserData);
		int eAIN(int Handle, int ChannelP, int ChannelN,
				DoubleByReference Voltage, int Range, int Resolution,
				int Settling, int Binary,  int Reserved1, int Reserved2);
		int eDAC(int Handle, int Channel, double Voltage, int Binary,
				int Reserved1, int Reserved2);
		int eDI(int Handle, int Channel, IntByReference State);
		int eDO(int Handle, int Channel, int State);
		int eAddGoGet(int Handle, int NumRequests, int[] aIOTypes,
				int[] aChannels, double[] aValues, int[] ax1s,
				int[] aRequestErrors, IntByReference GoError,
				int[] aResultErrors);
		int eTCConfig(int Handle, int[] aEnableTimers, int[] aEnableCounters,
				int TCPinOffset, int TimerClockBaseIndex,
				int TimerClockDivisor, int[] aTimerModes,
				double[] aTimerValues, int Reserved1, int Reserved2);
		int eTCValues(int Handle, int[] aReadTimers, int[] aUpdateResetTimers,
				int[] aReadCounters, int[] aResetCounters,
				double[] aTimerValues, double[] aCounterValues, int Reserved1,
				int Reserved2);
		int eModbus(int Handle, int readwrite, int addr, int size,
				byte[] value); //unsigned char
		int ResetLabJack(int Handle);
		int GetNextError(int Handle, IntByReference pIOType,
				IntByReference pChannel);
		int GetStreamError(int Handle);
		int DoubleToStringAddress(double Number, Pointer pString, int HexDot);
		int StringToDoubleAddress(final String pString,
				DoubleByReference pNumber, int HexDot);
		int StringToConstant(final String pString);
		void ErrorToString(int ErrorCode, Pointer pString);
		double GetDriverVersion();
		int GetThreadID(); //unsigned int return
		int TCVoltsToTemp(int TCType, double TCVolts, double CJTempK,
				DoubleByReference pTCTempK);
	}
	
	/**
	 * Constants is a class containing the constants from the LJUD driver.
	 * The "LJ_" prefix have been removed from the original names.
	 */
	public class Constants {
		private Constants() {}

		// Device types:
		public static final int dtUE9 = 9;
		public static final int dtU3 = 3;
		public static final int dtU6 = 6;
		public static final int dtSMB = 1000;

		// Connection types:
		public static final int ctUSB = 1; // UE9 + U3 + U6
		public static final int ctETHERNET = 2; // UE9 only
		public static final int ctETHERNET_MB = 3; // Modbus over Ethernet.  UE9 only.
		public static final int ctETHERNET_DATA_ONLY = 4; // Opens data port but not stream port.  UE9 only.
		public static final int ctUSB_RAW = 101; // UE9 + U3 + U6
		public static final int ctETHERNET_RAW = 102; // UE9 only

		// IO types:
		public static final int ioGET_AIN = 10; // UE9 + U3 + U6.
		public static final int ioGET_AIN_DIFF = 15; // U3/U6 only.
		public static final int ioGET_AIN_ADVANCED = 16; // For testing purposes. 

		public static final int ioPUT_AIN_RANGE = 2000; // UE9 + U6
		public static final int ioGET_AIN_RANGE = 2001; // UE9 only

		public static final int ioPUT_ANALOG_ENABLE_BIT = 2013; // U3 only
		public static final int ioGET_ANALOG_ENABLE_BIT = 2014; // U3 only
		public static final int ioPUT_ANALOG_ENABLE_PORT = 2015; // U3 only
		public static final int ioGET_ANALOG_ENABLE_PORT = 2016; // U3 only

		public static final int ioPUT_DAC = 20; // UE9 + U3 + U6
		public static final int ioPUT_DAC_ENABLE = 2002; // UE9 + U3 (U3 on Channel 1 only)
		public static final int ioGET_DAC_ENABLE = 2003; // UE9 + U3 (U3 on Channel 1 only)

		public static final int ioGET_DIGITAL_BIT = 30; // UE9 + U3 + U6.
		public static final int ioGET_DIGITAL_BIT_DIR = 31; // UE9 + U3 + U6
		public static final int ioGET_DIGITAL_BIT_STATE = 32; // UE9 + U3 + U6.

		public static final int ioGET_DIGITAL_PORT = 35; // UE9 + U3 + U6
		public static final int ioGET_DIGITAL_PORT_DIR = 36; // UE9 + U3 + U6
		public static final int ioGET_DIGITAL_PORT_STATE = 37; //UE9 + U3 + U6

		public static final int ioPUT_DIGITAL_BIT = 40; // UE9 + U3 + U6

		public static final int ioPUT_DIGITAL_PORT = 45; // UE9 + U3 + U6

		public static final int ioPUT_WAIT = 70; // U3 + U6

		public static final int ioGET_COUNTER = 50; // UE9 + U3 + U6

		public static final int ioPUT_COUNTER_ENABLE = 2008; // UE9 + U3 + U6
		public static final int ioGET_COUNTER_ENABLE = 2009; // UE9 + U3 + U6

		public static final int ioPUT_COUNTER_RESET = 2012;  // UE9 + U3 + U6

		public static final int ioGET_TIMER = 60; // UE9 + U3 + U6

		public static final int ioPUT_TIMER_VALUE = 2006; // UE9 + U3 + U6.
		public static final int ioPUT_TIMER_MODE = 2004; // UE9 + U3 + U6.
		public static final int ioGET_TIMER_MODE = 2005; // UE9 + U3 + U6

		public static final int ioSHT_GET_READING = 500; // UE9 + U3 + U6

		public static final int ioSPI_COMMUNICATION = 503; // UE9 + U3 + U6

		public static final int ioI2C_COMMUNICATION = 504; // UE9 + U3 + U6
		public static final int ioASYNCH_COMMUNICATION = 505; // UE9 + U3 + U6
		public static final int ioTDAC_COMMUNICATION = 506; // UE9 + U3 + U6

		public static final int ioPIN_CONFIGURATION_RESET = 2017; // UE9 + U3 + U6

		public static final int ioRAW_OUT = 100; // UE9 + U3 + U6
		public static final int ioRAW_IN = 101; // UE9 + U3 + U6

		public static final int ioRAWMB_OUT = 104; // UE9 Only
		public static final int ioRAWMB_IN = 105;  // UE9 only

		public static final int ioSET_DEFAULTS = 103; // U3 + U6

		public static final int ioADD_STREAM_CHANNEL = 200; // UE9 + U3 + U6
		public static final int ioADD_STREAM_CHANNEL_DIFF = 206; // U3 + U6

		public static final int ioCLEAR_STREAM_CHANNELS = 201; // UE9 + U3 + U6
		public static final int ioSTART_STREAM = 202; // UE9 + U3 + U6
		public static final int ioSTOP_STREAM = 203; // UE9 + U3 + U6
		 
		public static final int ioADD_STREAM_DAC = 207; //UE9 only
		public static final int ioGET_STREAM_DATA = 204; // UE9 + U3 + U6

		//Callbacks not tested in Java
		
		//typedef void (*tStreamCallback)(long ScansAvailable, long UserValue);
		public static final int ioSET_STREAM_CALLBACK = 205; // UE9 + U3 + U6
		public static final int ioSET_STREAM_CALLBACK_PTR = 260; // UE9 + U3 + U6. This is for 64-bit compatibility.

		public static final int ioBUZZER = 300; // U3 only

		//typedef void (*tEventCallback)(long EventCode, long Data1, long Data2, long Data3, long UserValue);
		public static final int ioSET_EVENT_CALLBACK = 400; // UE9 + U3 + U6

		// These are the possible EventCodes that will be passed:
		public static final int ecDISCONNECT = 1;
		public static final int ecRECONNECT = 2;
		public static final int ecSTREAMERROR = 4;
		// Future events will be power of 2.

		// Config iotypes:
		public static final int ioPUT_CONFIG = 1000; // UE9 + U3 + U6
		public static final int ioGET_CONFIG = 1001; // UE9 + U3 + U6

		// Channel numbers used for CONFIG types:
		public static final int chLOCALID = 0; // UE9 + U3 + U6
		public static final int chHARDWARE_VERSION = 10; // UE9 + U3 + U6 (Read Only)
		public static final int chSERIAL_NUMBER = 12; // UE9 + U3 + U6 (Read Only)
		public static final int chFIRMWARE_VERSION = 11; // UE9 + U3 + U6 (Read Only)
		public static final int chBOOTLOADER_VERSION = 15; // UE9 + U3 + U6 (Read Only)
		public static final int chPRODUCTID = 8; // UE9 + U3 + U6 (Read Only)

		// UE9 specific:
		public static final int chCOMM_POWER_LEVEL = 1; // UE9
		public static final int chIP_ADDRESS = 2; // UE9
		public static final int chGATEWAY = 3; // UE9
		public static final int chSUBNET = 4; // UE9
		public static final int chPORTA = 5; // UE9
		public static final int chPORTB = 6; // UE9
		public static final int chDHCP = 7; // UE9
		public static final int chMACADDRESS = 9; // UE9
		public static final int chCOMM_FIRMWARE_VERSION = 11; // UE9
		public static final int chCONTROL_POWER_LEVEL = 13; // UE9
		public static final int chCONTROL_FIRMWARE_VERSION = 14; // UE9 (Read Only)
		public static final int chCONTROL_BOOTLOADER_VERSION = 15; // UE9 (Read Only)
		public static final int chCONTROL_RESET_SOURCE = 16; // UE9 (Read Only)
		public static final int chUE9_PRO = 19; // UE9 (Read Only)

		public static final int chLED_STATE = 17; // U3 + U6
		public static final int chSDA_SCL = 18; // U3 + U6

		public static final int chU3HV = 22; // U3 only (Read Only)

		public static final int chU6_PRO = 23; // U6 only.

		// Driver related:
		public static final int chCOMMUNICATION_TIMEOUT = 20;
		public static final int chSTREAM_COMMUNICATION_TIMEOUT = 21;

		public static final int chCAL_CONSTANTS = 400; // UE9 + U3 + U6
		public static final int chUSER_MEM = 402; // UE9 + U3 + U6

		public static final int chUSB_STRINGS = 404; // U3 + U6

		// Timer/counter related:
		public static final int chNUMBER_TIMERS_ENABLED = 1000; // UE9 + U3 + U6
		public static final int chTIMER_CLOCK_BASE = 1001; // UE9 + U3 + U6
		public static final int chTIMER_CLOCK_DIVISOR = 1002; // UE9 + U3 + U6
		public static final int chTIMER_COUNTER_PIN_OFFSET = 1003; // U3 + U6

		// AIn related:
		public static final int chAIN_RESOLUTION = 2000; // UE9 + U3 + U6
		public static final int chAIN_SETTLING_TIME = 2001; // UE9 + U3 + U6
		public static final int chAIN_BINARY = 2002; // UE9 + U3 + U6

		// DAC related:
		public static final int chDAC_BINARY = 3000; // UE9 + U3

		// SHT related:
		public static final int chSHT_TEMP = 5000; // UE9 + U3 + U6
		public static final int chSHT_RH = 5001; // UE9 + U3 + U6
		public static final int chSHT_DATA_CHANNEL = 5002; // UE9 + U3 + U6
		public static final int chSHT_CLOCK_CHANNEL = 5003; // UE9 + U3 + U6

		// SPI related:
		public static final int chSPI_AUTO_CS = 5100; // UE9 + U3 + U6
		public static final int chSPI_DISABLE_DIR_CONFIG = 5101; // UE9 + U3 + U6
		public static final int chSPI_MODE = 5102; // UE9 + U3 + U6
		public static final int chSPI_CLOCK_FACTOR = 5103; // UE9 + U3 + U6
		public static final int chSPI_MOSI_PIN_NUM = 5104; // UE9 + U3 + U6
		public static final int chSPI_MISO_PIN_NUM = 5105; // UE9 + U3 + U6
		public static final int chSPI_CLK_PIN_NUM = 5106; // UE9 + U3 + U6
		public static final int chSPI_CS_PIN_NUM = 5107; // UE9 + U3 + U6

		// I2C related:
		public static final int chI2C_ADDRESS_BYTE = 5108; // UE9 + U3 + U6
		public static final int chI2C_SCL_PIN_NUM = 5109; // UE9 + U3 + U6
		public static final int chI2C_SDA_PIN_NUM = 5110; // UE9 + U3 + U6
		public static final int chI2C_OPTIONS = 5111; // UE9 + U3 + U6
		public static final int chI2C_SPEED_ADJUST = 5112; // UE9 + U3 + U6
		public static final int chI2C_READ = 5113; // UE9 + U3 + U6
		public static final int chI2C_WRITE = 5114; // UE9 + U3 + U6
		public static final int chI2C_GET_ACKS = 5115; // UE9 + U3 + U6
		public static final int chI2C_WRITE_READ = 5130; // UE9 + U3 + U6

		// ASYNCH related:
		public static final int chASYNCH_RX = 5117; // UE9 + U3 + U6
		public static final int chASYNCH_TX = 5118; // UE9 + U3 + U6
		public static final int chASYNCH_FLUSH = 5128; // UE9 + U3 + U6
		public static final int chASYNCH_ENABLE = 5129; // UE9 + U3 + U6

		public static final int chASYNCH_BAUDFACTOR = 5127; // UE9 + U3 + U6

		// LJ TickDAC related:
		public static final int chTDAC_SCL_PIN_NUM = 5119; // UE9 + U3 + U6
		public static final int chTDAC_SERIAL_NUMBER = 5120; // UE9 + U3 + U6: Read only.
		public static final int chTDAC_READ_USER_MEM = 5121; // UE9 + U3 + U6
		public static final int chTDAC_WRITE_USER_MEM = 5122; // UE9 + U3 + U6
		public static final int chTDAC_READ_CAL_CONSTANTS = 5123; // UE9 + U3 + U6
		public static final int chTDAC_WRITE_CAL_CONSTANTS = 5124; // UE9 + U3 + U6
		public static final int chTDAC_UPDATE_DACA = 5125; // UE9 + U3 + U6
		public static final int chTDAC_UPDATE_DACB = 5126; // UE9 + U3 + U6

		// Stream related.
		public static final int chSTREAM_SCAN_FREQUENCY = 4000; // UE9 + U3 + U6
		public static final int chSTREAM_BUFFER_SIZE = 4001;
		public static final int chSTREAM_CLOCK_OUTPUT = 4002; // UE9 only
		public static final int chSTREAM_EXTERNAL_TRIGGER = 4003; // UE9 only
		public static final int chSTREAM_WAIT_MODE = 4004;
		public static final int chSTREAM_DISABLE_AUTORECOVERY = 4005; // U3 + U6
		public static final int chSTREAM_SAMPLES_PER_PACKET = 4108; // UE9 + U3 + U6. Read only for UE9.
		public static final int chSTREAM_READS_PER_SECOND = 4109;
		public static final int chAIN_STREAM_SETTLING_TIME = 4110; // U6 only

		// Read only stream related
		public static final int chSTREAM_BACKLOG_COMM = 4105; // UE9 + U3 + U6
		public static final int chSTREAM_BACKLOG_CONTROL = 4106; // UE9 only
		public static final int chSTREAM_BACKLOG_UD = 4107;

		// Special channel numbers
		public static final int chALL_CHANNELS = -1;
		public static final int INVALID_CONSTANT = -999;

		//Thermocouple Type constants.
		public static final int ttB = 6001;
		public static final int ttE = 6002;
		public static final int ttJ = 6003;
		public static final int ttK = 6004;
		public static final int ttN = 6005;
		public static final int ttR = 6006;
		public static final int ttS = 6007;
		public static final int ttT = 6008;

		// Other constants:

		// Ranges (not all are supported by all devices):
		public static final int rgBIP20V = 1;
		public static final int rgBIP10V = 2;
		public static final int rgBIP5V = 3;
		public static final int rgBIP4V = 4;
		public static final int rgBIP2P5V = 5;
		public static final int rgBIP2V = 6;
		public static final int rgBIP1P25V = 7;
		public static final int rgBIP1V = 8;
		public static final int rgBIPP625V = 9;
		public static final int rgBIPP1V = 10;
		public static final int rgBIPP01V = 11;

		public static final int rgUNI20V = 101;
		public static final int rgUNI10V = 102;
		public static final int rgUNI5V = 103;
		public static final int rgUNI4V = 104;
		public static final int rgUNI2P5V = 105;
		public static final int rgUNI2V = 106;
		public static final int rgUNI1P25V = 107;
		public static final int rgUNI1V = 108;
		public static final int rgUNIP625V = 109;
		public static final int rgUNIP5V = 110;
		public static final int rgUNIP25V = 112;
		public static final int rgUNIP3125V = 111;
		public static final int rgUNIP025V = 113;
		public static final int rgUNIP0025V = 114;

		// Timer modes:
		public static final int tmPWM16 = 0;
		public static final int tmPWM8 = 1;
		public static final int tmRISINGEDGES32 = 2;
		public static final int tmFALLINGEDGES32 = 3;
		public static final int tmDUTYCYCLE = 4;
		public static final int tmFIRMCOUNTER = 5;
		public static final int tmFIRMCOUNTERDEBOUNCE = 6;
		public static final int tmFREQOUT = 7;
		public static final int tmQUAD = 8;
		public static final int tmTIMERSTOP = 9;
		public static final int tmSYSTIMERLOW = 10;
		public static final int tmSYSTIMERHIGH = 11;
		public static final int tmRISINGEDGES16 = 12;
		public static final int tmFALLINGEDGES16 = 13;
		public static final int tmLINETOLINE = 14;

		// Timer clocks:
		public static final int tc750KHZ = 0; // UE9: 750 kHz 
		public static final int tcSYS = 1;    // UE9 + U3 + U6: System clock

		public static final int tc2MHZ = 10;       // U3: Hardware Version 1.20 or lower
		public static final int tc6MHZ = 11;       // U3: Hardware Version 1.20 or lower
		public static final int tc24MHZ = 12;      // U3: Hardware Version 1.20 or lower
		public static final int tc500KHZ_DIV = 13; // U3: Hardware Version 1.20 or lower
		public static final int tc2MHZ_DIV = 14;   // U3: Hardware Version 1.20 or lower
		public static final int tc6MHZ_DIV = 15;   // U3: Hardware Version 1.20 or lower
		public static final int tc24MHZ_DIV = 16;  // U3: Hardware Version 1.20 or lower

		public static final int tc4MHZ = 20;      // U3: Hardware Version 1.21 or higher, + U6
		public static final int tc12MHZ = 21;     // U3: Hardware Version 1.21 or higher, + U6
		public static final int tc48MHZ = 22;     // U3: Hardware Version 1.21 or higher, + U6
		public static final int tc1MHZ_DIV = 23;  // U3: Hardware Version 1.21 or higher, + U6
		public static final int tc4MHZ_DIV = 24;  // U3: Hardware Version 1.21 or higher, + U6
		public static final int tc12MHZ_DIV = 25; // U3: Hardware Version 1.21 or higher, + U6
		public static final int tc48MHZ_DIV = 26; // U3: Hardware Version 1.21 or higher, + U6

		// Stream wait modes:
		public static final int swNONE = 1;
		public static final int swALL_OR_NONE = 2;
		public static final int swPUMP = 11;
		public static final int swSLEEP = 12;

		// BETA CONSTANTS:
		// SWDT:
		public static final int ioSWDT_CONFIG = 507; // UE9 + U3 + U6
		public static final int ioSWDT_STROKE = 508; // UE9 only 

		public static final int chSWDT_ENABLE = 5200; // UE9 + U3 + U6
		public static final int chSWDT_DISABLE = 5201; // UE9 + U3 + U6

		// Used with LJ_io_PUT_CONFIG
		public static final int chSWDT_RESET_DEVICE = 5202; // U3 + U6.  Write only.
		public static final int chSWDT_RESET_COMM = 5203; // UE9 only.  Write only.
		public static final int chSWDT_RESET_CONTROL = 5204; // UE9 only.  Write only.
		public static final int chSWDT_UPDATE_DIOA = 5205; // UE9 + U3 + U6.  Write only.
		public static final int chSWDT_UPDATE_DIOB = 5206; // UE9 only.  Write only.
		public static final int chSWDT_DIOA_CHANNEL = 5207; // UE9 + U3 + U6.  Write only.
		public static final int chSWDT_DIOA_STATE = 5208; // UE9 + U3 + U6.  Write only.
		public static final int chSWDT_DIOB_CHANNEL = 5209; // UE9.  Write only.
		public static final int chSWDT_DIOB_STATE = 5210; // UE9.  Write only.
		public static final int chSWDT_UPDATE_DAC0 = 5211; // UE9 only.  Write only.
		public static final int chSWDT_UPDATE_DAC1 = 5212; // UE9 only.  Write only.
		public static final int chSWDT_DAC0 = 5213; // UE9 only.  Write only.
		public static final int chSWDT_DAC1 = 5214; // UE9 only.  Write only.
		public static final int chSWDT_DAC_ENABLE = 5215; // UE9 only.  Write only.
		public static final int chSWDT_STRICT_ENABLE = 5216; // UE9 only
		public static final int chSWDT_INITIAL_ROLL_TIME = 5217; // UE9 only

		// Depreciated constants:
		public static final int ioANALOG_INPUT = 10;
		public static final int ioANALOG_OUTPUT = 20; // UE9 + U3
		public static final int ioDIGITAL_BIT_IN = 30; // UE9 + U3
		public static final int ioDIGITAL_PORT_IN = 35; // UE9 + U3
		public static final int ioDIGITAL_BIT_OUT = 40; // UE9 + U3
		public static final int ioDIGITAL_PORT_OUT = 45; // UE9 + U3
		public static final int ioCOUNTER = 50; // UE9 + U3
		public static final int ioTIMER = 60; // UE9 + U3
		public static final int ioPUT_COUNTER_MODE = 2010; // UE9
		public static final int ioGET_COUNTER_MODE = 2011; // UE9
		public static final int ioGET_TIMER_VALUE = 2007; // UE9
		public static final int ioCYCLE_PORT = 102;  // UE9 
		public static final int chTIMER_CLOCK_CONFIG = 1001; // UE9 + U3
		public static final int ioPUT_CAL_CONSTANTS = 400;
		public static final int ioGET_CAL_CONSTANTS = 401;
		public static final int ioPUT_USER_MEM = 402;
		public static final int ioGET_USER_MEM = 403;
		public static final int ioPUT_USB_STRINGS = 404;
		public static final int ioGET_USB_STRINGS = 405;
		public static final int ioSHT_DATA_CHANNEL = 501; // UE9 + U3
		public static final int ioSHT_CLOCK_CHANNEL = 502; // UE9 + U3
		public static final int chI2C_ADDRESS = 5108; // UE9 + U3
		public static final int chASYNCH_CONFIG = 5116; // UE9 + U3
		public static final int rgUNIP500V = 110; // 0V to +0.500V
		public static final int ioENABLE_POS_PULLDOWN = 2018; // U6
		public static final int ioENABLE_NEG_PULLDOWN = 2019; // U6
		public static final int rgAUTO = 0;
		public static final int chSWDT_UDPATE_DIOA = 5205;
	}
	
	/**
	 * Errors is an enum containing the error constants from the LJUD driver.
	 * The "LJE_" prefix have been removed from the original names.
	 */
	public enum Errors {
		NOERROR(0),
		COMMAND_LIST_ERROR(1), 
		INVALID_CHANNEL_NUMBER(2),
		INVALID_RAW_INOUT_PARAMETER(3),
		UNABLE_TO_START_STREAM(4),
		UNABLE_TO_STOP_STREAM(5),
		NOTHING_TO_STREAM(6),
		UNABLE_TO_CONFIG_STREAM(7),
		BUFFER_OVERRUN(8),
		STREAM_NOT_RUNNING(9),
		INVALID_PARAMETER(10),
		INVALID_STREAM_FREQUENCY(11),
		INVALID_AIN_RANGE(12),
		STREAM_CHECKSUM_ERROR(13),
		STREAM_COMMAND_ERROR(14),
		STREAM_ORDER_ERROR(15),
		AD_PIN_CONFIGURATION_ERROR(16),
		REQUEST_NOT_PROCESSED(17),

		SCRATCH_ERROR(19),
		DATA_BUFFER_OVERFLOW(20),
		ADC0_BUFFER_OVERFLOW(21),
		FUNCTION_INVALID(22),
		SWDT_TIME_INVALID(23),
		FLASH_ERROR(24),
		STREAM_IS_ACTIVE(25),
		STREAM_TABLE_INVALID(26),
		STREAM_CONFIG_INVALID(27),
		STREAM_BAD_TRIGGER_SOURCE(28),
		STREAM_INVALID_TRIGGER(30),
		STREAM_ADC0_BUFFER_OVERFLOW(31),
		STREAM_SAMPLE_NUM_INVALID(33),
		STREAM_BIPOLAR_GAIN_INVALID(34),
		STREAM_SCAN_RATE_INVALID(35),
		TIMER_INVALID_MODE(36),
		TIMER_QUADRATURE_AB_ERROR(37),
		TIMER_QUAD_PULSE_SEQUENCE(38),
		TIMER_BAD_CLOCK_SOURCE(39),
		TIMER_STREAM_ACTIVE(40),
		TIMER_PWMSTOP_MODULE_ERROR(41),
		TIMER_SEQUENCE_ERROR(42),
		TIMER_SHARING_ERROR(43),
		TIMER_LINE_SEQUENCE_ERROR(44),
		EXT_OSC_NOT_STABLE(45),
		INVALID_POWER_SETTING(46),
		PLL_NOT_LOCKED(47),
		INVALID_PIN(48),
		IOTYPE_SYNCH_ERROR(49),
		INVALID_OFFSET(50),
		FEEDBACK_IOTYPE_NOT_VALID(51),
		CANT_CONFIGURE_PIN_FOR_ANALOG(67),
		CANT_CONFIGURE_PIN_FOR_DIGITAL(68),
		TC_PIN_OFFSET_MUST_BE_4_TO_8(70),
		INVALID_DIFFERENTIAL_CHANNEL(71),
		DSP_SIGNAL_OUT_OF_RANGE(72),

		// Other errors
		SHT_CRC(52),
		SHT_MEASREADY(53),
		SHT_ACK(54),
		SHT_SERIAL_RESET(55),
		SHT_COMMUNICATION(56),

		AIN_WHILE_STREAMING(57),

		STREAM_TIMEOUT(58),
		STREAM_CONTROL_BUFFER_OVERFLOW(59),
		STREAM_SCAN_OVERLAP(60),
		FIRMWARE_VERSION_IOTYPE(61),
		FIRMWARE_VERSION_CHANNEL(62),
		FIRMWARE_VERSION_VALUE(63),
		HARDWARE_VERSION_IOTYPE(64),
		HARDWARE_VERSION_CHANNEL(65),
		HARDWARE_VERSION_VALUE(66),

		LJTDAC_ACK_ERROR(69),

		STREAM_INVALID_CONNECTION(73),

		MIN_GROUP_ERROR(1000), // All errors above this number will stop all requests, below this number are request level errors.

		UNKNOWN_ERROR(1001),
		INVALID_DEVICE_TYPE(1002),
		INVALID_HANDLE(1003),
		DEVICE_NOT_OPEN(1004),
		NO_DATA_AVAILABLE(1005),
		NO_MORE_DATA_AVAILABLE(1006),
		LABJACK_NOT_FOUND(1007),
		COMM_FAILURE(1008),
		CHECKSUM_ERROR(1009),
		DEVICE_ALREADY_OPEN(1010),
		COMM_TIMEOUT(1011),
		USB_DRIVER_NOT_FOUND(1012),
		INVALID_CONNECTION_TYPE(1013),
		INVALID_MODE(1014),
		DEVICE_NOT_CONNECTED(1015),

		DISCONNECT(2000), 
		RECONNECT(2001),

		MIN_USER_ERROR(3000),
		MAX_USER_ERROR(3999),

		// Warning are negative
		DEVICE_NOT_CALIBRATED(-1),
		UNABLE_TO_READ_CALDATA(-2);

		private final int value;
		
		private Errors(int value) {
			this.value = value;
		};
		
		/**
		 * Gets the error's values.
		 */
		public int getValue() {
			return value;
		};
	} 

	/* Helper methods */
	private static void handleError(int error) {
		if(error > LJUD.Errors.NOERROR.getValue()) {
			throw new LJUDException(error);
		}
		//Warning error codes (< 0) do not cause an exception. The warning
		//error code will be returned by the wrapper function however.
	}
	
	/* LJUD wrapper methods */

	public static void close() {
		LabJackUD.INSTANCE.Close();
	}

	public static int listAll(int deviceType, int connectionType,
			IntByReference pNumFound, int[] pSerialNumbers, int[] pIDs,
			double[] pAddresses) {
		int error = LabJackUD.INSTANCE.ListAll(deviceType, connectionType,
				pNumFound, pSerialNumbers, pIDs, pAddresses);
		handleError(error);
		return error;
	}
	
	public static int listAllS(final String pDeviceType, final String pConnectionType,
			IntByReference pNumFound, int[] pSerialNumbers, int[] pIDs,
			double[] pAddresses) {
		int error = LabJackUD.INSTANCE.ListAllS(pDeviceType, pConnectionType,
				pNumFound, pSerialNumbers, pIDs, pAddresses);
		handleError(error);
		return error;
	}

	public static int openLabJack(int deviceType, int connectionType,
			final String pAddress, int firstFound, IntByReference pHandle) {
		int error = LabJackUD.INSTANCE.OpenLabJack(deviceType, connectionType,
				pAddress, firstFound, pHandle);
		handleError(error);
		return error;
	}

	public static int openLabJackS(final String pDeviceType, final String pConnectionType,
			final String pAddress, int firstFound, IntByReference pHandle) {
		int error = LabJackUD.INSTANCE.OpenLabJackS(pDeviceType,
				pConnectionType, pAddress, firstFound, pHandle);
		handleError(error);
		return error;
	}

	public static int addRequest(int handle, int ioType, int channel,
			double value, int x1, double userData) {
		int error = LabJackUD.INSTANCE.AddRequest(handle, ioType, channel,
				value, x1, userData);
		handleError(error);
		return error;
	}
	
	public static int addRequestS(int handle, final String pIOType,
			int channel, double value, int x1, double userData) {
		int error = LabJackUD.INSTANCE.AddRequestS(handle, pIOType, channel,
				value, x1, userData);
		handleError(error);
		return error;
	}

	public static int addRequestSS(int handle, final String ioType,
			final String pChannel, double value, int x1, double userData) {
		int error = LabJackUD.INSTANCE.AddRequestSS(handle, ioType, pChannel,
				value, x1, userData);
		handleError(error);
		return error;
	}
		
	public static int addRequestPtr(int handle, int ioType, int channel,
			double value, Pointer x1, double userData) {
		int error = LabJackUD.INSTANCE.AddRequestPtr(handle, ioType, channel,
				value, x1, userData);
		handleError(error);
		return error;
	}

	public static int go() {
		int error = LabJackUD.INSTANCE.Go();
		handleError(error);
		return error;
	}

	public static int goOne(int handle) {
		int error = LabJackUD.INSTANCE.GoOne(handle);
		handleError(error);
		return error;
	}
	
	public static int eGet(int handle, int ioType, int channel,
			DoubleByReference pValue, int x1) {
		int error = LabJackUD.INSTANCE.eGet(handle, ioType, channel, pValue,
				x1);
		handleError(error);
		return error;
	}

	public static int eGet(int handle, int ioType, int channel,
			DoubleByReference pValue, byte[] x1) {
		return eGet_U8Array(handle, ioType, channel, pValue, x1);
	}

	public static int eGet(int handle, int ioType, int channel,
			DoubleByReference pValue, double[] x1) {
		return eGet_DblArray(handle, ioType, channel, pValue, x1);
	}

	public static int eGetPtr(int handle, int ioType, int channel,
			DoubleByReference pValue, Pointer x1) {
		int error = LabJackUD.INSTANCE.eGetPtr(handle, ioType, channel, pValue,
				x1);
		handleError(error);
		return error;
	}

	public static int eGetS(int handle, final String pIOType, int channel,
			DoubleByReference pValue, int x1) {
		int error = LabJackUD.INSTANCE.eGetS(handle, pIOType, channel, pValue,
				x1);
		handleError(error);
		return error;
	}

	public static int eGetSS(int handle, final String pIOType,
			final String pChannel, DoubleByReference pValue, int x1) {
		int error = LabJackUD.INSTANCE.eGetSS(handle, pIOType, pChannel,
				pValue, x1);
		handleError(error);
		return error;
	}
	
	public static int ePut(int handle, int ioType, int channel, double value,
			int x1) {
		int error = LabJackUD.INSTANCE.ePut(handle, ioType, channel, value,
				x1);
		handleError(error);
		return error;
	}

	public static int ePut(int handle, int ioType, int channel, double value,
			byte[] x1) {
		DoubleByReference pValue = new DoubleByReference(value);
		int error = eGet_U8Array(handle, ioType, channel, pValue, x1);
		return error;
	}

	public static int ePut(int handle, int ioType, int channel, double value,
			double[] x1) {
		DoubleByReference pValue = new DoubleByReference(value);
		int error = eGet_DblArray(handle, ioType, channel, pValue, x1);
		return error;
	}
	
	public static int ePutS(int handle, final String pIOType, int channel,
			double value, int x1) {
		int error = LabJackUD.INSTANCE.ePutS(handle, pIOType, channel, value,
				x1);
		handleError(error);
		return error;
	}

	public static int ePutSS(int handle, final String pIOType, final String pChannel,
			double value, int x1) {
		int error = LabJackUD.INSTANCE.ePutSS(handle, pIOType, pChannel, value,
				x1);
		handleError(error);
		return error;
	}

	public static int eGet_DblArray(int handle, int ioType, int channel,
			DoubleByReference pValue, double[] x1) {
		int error = LabJackUD.INSTANCE.eGet_DblArray(handle, ioType, channel,
				pValue, x1);
		handleError(error);
		return error;
	}
	
	public static int eGet_U8Array(int handle, int ioType, int channel,
			DoubleByReference pValue, byte[] x1) {
		int error = LabJackUD.INSTANCE.eGet_U8Array(handle, ioType, channel,
				pValue, x1);
		handleError(error);
		return error;
	}
	
	public static int eGetS_DblArray(int handle, final String pIOType,
			int channel, DoubleByReference pValue, double[] x1) {
		int error = LabJackUD.INSTANCE.eGetS_DblArray(handle, pIOType, channel,
				pValue, x1);
		handleError(error);
		return error;
	}

	public static int eGetS_U8Array(int handle, final String pIOType,
			int channel, DoubleByReference pValue, byte[] x1) {
		int error = LabJackUD.INSTANCE.eGetS_U8Array(handle, pIOType, channel,
				pValue, x1);
		handleError(error);
		return error;
	}

	public static int eGetSS_DblArray(int handle, final String pIOType,
			final String pChannel, DoubleByReference pValue, double[] x1) {
		int error = LabJackUD.INSTANCE.eGetSS_DblArray(handle, pIOType,
				pChannel, pValue, x1);
		handleError(error);
		return error;
	}

	public static int eGetSS_U8Array(int handle, final String pIOType,
			final String pChannel, DoubleByReference pValue, byte[] x1) {
		int error = LabJackUD.INSTANCE.eGetSS_U8Array(handle, pIOType,
				pChannel, pValue, x1);
		handleError(error);
		return error;
	}
	
	public static int getResult(int handle, int ioType, int channel,
			DoubleByReference pValue) {
		int error = LabJackUD.INSTANCE.GetResult(handle, ioType, channel,
				pValue);
		handleError(error);
		return error;
	}

	public static int getResultS(int handle, final String pIOType, int channel,
			DoubleByReference pValue) {
		int error = LabJackUD.INSTANCE.GetResultS(handle, pIOType, channel,
				pValue);
		handleError(error);
		return error;
	}

	public static int getResultSS(int handle, final String pIOType,
			final String pChannel, DoubleByReference pValue) {
		int error = LabJackUD.INSTANCE.GetResultSS(handle, pIOType, pChannel,
				pValue);
		handleError(error);
		return error;
	}

	public static int getFirstResult(int handle, IntByReference pIOType,
			IntByReference pChannel, DoubleByReference pValue,
			IntByReference px1, DoubleByReference pUserData) {
		int error = LabJackUD.INSTANCE.GetFirstResult(handle, pIOType,
				pChannel, pValue, px1, pUserData);
		handleError(error);
		return error;
	}
	
	public static int getNextResult(int handle, IntByReference pIOType,
			IntByReference pChannel, DoubleByReference pValue,
			IntByReference px1, DoubleByReference pUserData) {
		int error = LabJackUD.INSTANCE.GetNextResult(handle, pIOType, pChannel,
				pValue, px1, pUserData);
		handleError(error);
		return error;
	}

	public static int eAIN(int handle, int channelP, int channelN,
			DoubleByReference voltage, int range, int resolution, int settling,
			int binary,  int reserved1, int reserved2) {
		int error = LabJackUD.INSTANCE.eAIN(handle, channelP, channelN,
				voltage, range, resolution, settling, binary, reserved1,
				reserved2);
		handleError(error);
		return error;
	}

	public static int eDAC(int handle, int channel, double voltage, int binary,
			int reserved1, int reserved2) {
		int error = LabJackUD.INSTANCE.eDAC(handle, channel, voltage, binary,
				reserved1, reserved2);
		handleError(error);
		return error;
	}

	
	public static int eDI(int handle, int channel, IntByReference state) {
		int error = LabJackUD.INSTANCE.eDI(handle, channel, state);
		handleError(error);
		return error;
	}
	
	public static int eDO(int handle, int channel, int state)  {
		int error = LabJackUD.INSTANCE.eDO(handle, channel, state);
		handleError(error);
		return error;
	}
	
	public static int eAddGoGet(int handle, int numRequests, int[] aIOTypes,
			int[] aChannels, double[] aValues, int[] ax1s,
			int[] aRequestErrors, IntByReference goError,
			int[] aResultErrors) {
		return LabJackUD.INSTANCE.eAddGoGet(handle, numRequests, aIOTypes,
				aChannels, aValues, ax1s, aRequestErrors, goError,
				aResultErrors);
	}
	
	public static int eTCConfig(int handle, int[] aEnableTimers,
			int[] aEnableCounters, int tcPinOffset, int timerClockBaseIndex,
			int timerClockDivisor, int[] aTimerModes, double[] aTimerValues,
			int reserved1, int reserved2) {
		int error = LabJackUD.INSTANCE.eTCConfig(handle, aEnableTimers,
				aEnableCounters, tcPinOffset, timerClockBaseIndex,
				timerClockDivisor, aTimerModes, aTimerValues, reserved1,
				reserved2);
		handleError(error);
		return error;
	}

	public static int eTCValues(int handle, int[] aReadTimers,
			int[] aUpdateResetTimers, int[] aReadCounters,
			int[] aResetCounters, double[] aTimerValues,
			double[] aCounterValues, int reserved1, int reserved2) {
		int error = LabJackUD.INSTANCE.eTCValues(handle, aReadTimers,
				aUpdateResetTimers, aReadCounters, aResetCounters,
				aTimerValues, aCounterValues, reserved1, reserved2);
		handleError(error);
		return error;
	}

	public static int eModbus(int handle, int readwrite, int addr, int size, byte[] value) {
		int error = LabJackUD.INSTANCE.eModbus(handle, readwrite, addr, size, value);
		handleError(error);
		return error;
	};

	public static int resetLabJack(int handle) {
		int error = LabJackUD.INSTANCE.ResetLabJack(handle);
		handleError(error);
		return error;
	}

	public static int getNextError(int handle, IntByReference pIOType, IntByReference pChannel) {
		return LabJackUD.INSTANCE.GetNextError(handle, pIOType, pChannel);
	}

	public static int getStreamError(int handle) {
		return LabJackUD.INSTANCE.GetStreamError(handle);
	}
		
	public static int doubleToStringAddress(double number, Pointer pString,
			int hexDot) {
		int error = LabJackUD.INSTANCE.DoubleToStringAddress(number, pString,
				hexDot);
		handleError(error);
		return error;
	}

	public static int stringToDoubleAddress(final String pString,
			DoubleByReference pNumber, int hexDot) {
		int error = LabJackUD.INSTANCE.StringToDoubleAddress(pString, pNumber, hexDot);
		handleError(error);
		return error;
	}
	
	public static int stringToConstant(final String pString) {
		return LabJackUD.INSTANCE.StringToConstant(pString);
	}

	public static void errorToString(int ErrorCode, Pointer pString) {
		LabJackUD.INSTANCE.ErrorToString(ErrorCode, pString);
	}

	public static double getDriverVersion() {
		return LabJackUD.INSTANCE.GetDriverVersion();
	}

	public static int getThreadID() {
		return LabJackUD.INSTANCE.GetThreadID();
	}
	
	public static int tcVoltsToTemp(int tcType, double tcVolts, double cjTempK, DoubleByReference pTCTempK) {
		int error = LabJackUD.INSTANCE.TCVoltsToTemp(tcType, tcVolts, cjTempK, pTCTempK);
		handleError(error);
		return error;
	}
	
	public LJUD() {}
}
